@@ -11,6 +11,8 @@
 #               3) Michael Meissner private communication
 #               4) Paul Samuelson private communication (S5)
 #               5) http://www.cybercom.net/~dcoffin/dcraw/
+#               6) http://forums.dpreview.com/forums/readflat.asp?forum=1012&thread=31350384
+#                  and http://forum.photome.de/viewtopic.php?f=2&t=353&p=742#p740
 #               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
@@ -21,10 +23,27 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.18';
+$VERSION = '1.23';
 
 sub ProcessFujiDir($$$);
 
+# the following RAF version numbers have been tested for writing:
+my %testedRAF = (
+    '0100' => 'E550 V1.00, E900 V1.00, S5600 V1.00, S6000fd V1.00, S6500fd V2.00',
+    '0104' => 'S5Pro V1.04',
+    '0106' => 'S5Pro V1.06',
+    '0111' => 'S5Pro V1.11',
+    '0114' => 'S9600 V1.00',
+    '0159' => 'S2Pro V1.00',
+    '0212' => 'S3Pro V2.12',
+    '0218' => 'S3Pro V2.18',
+    '0264' => 'F700  V2.00',
+    '0266' => 'S9500 V1.01',
+    '0269' => 'S9500 V1.02',
+    '0712' => 'S5000 V3.00',
+);
+
+# FujiFilm MakerNotes tags
 %Image::ExifTool::FujiFilm::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
@@ -231,6 +250,24 @@ sub ProcessFujiDir($$$);
 #        Name => 'ShutterCount',
 #        Writable => 'int16u',
 #    },
+    0x1033 => { #6
+        Name => 'EXRAuto',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Auto',
+            1 => 'Manual',
+        },
+    },
+    0x1034 => { #6
+        Name => 'EXRMode',
+        Writable => 'int16u',
+        PrintHex => 1,
+        PrintConv => {
+            0x100 => 'HR (High Resolution)',
+            0x200 => 'SN (Signal to Noise priority)',
+            0x300 => 'DR (Dynamic Range priority)',
+        },
+    },
     0x1100 => {
         Name => 'AutoBracketing',
         Writable => 'int16u',
@@ -315,7 +352,7 @@ sub ProcessFujiDir($$$);
             0x8000 => 'Film Simulation',
         },
     },
-    0x1403 => { #2
+    0x1403 => { #2 (only valid for manual DR, ref 6)
         Name => 'DevelopmentDynamicRange',
         Writable => 'int16u',
     },
@@ -335,6 +372,24 @@ sub ProcessFujiDir($$$);
         Name => 'MaxApertureAtMaxFocal',
         Writable => 'rational64s',
     },
+    # 0x1408 - values: '0100', 'S100', 'VQ10'
+    # 0x1409 - values: same as 0x1408
+    # 0x140a - values: 0, 1, 3, 5, 7
+    # 0x140b - DR value for AutoDR???? (ref 6) - values: 100
+    0x4100 => { #PH
+        Name => 'FacesDetected',
+        Writable => 'int16u',
+    },
+    0x4103 => { #PH
+        Name => 'FacePositions',
+        Writable => 'int16u',
+        Count => -1,
+        Notes => q{
+            left, top, right and bottom coordinates in full-sized image for each face
+            detected
+        },
+    },
+    # 0x4104 - also related to face detection (same number of entries as FacePositions)
     0x8000 => { #2
         Name => 'FileSource',
         Writable => 'string',
@@ -475,21 +530,12 @@ sub WriteRAF($$)
     # get the position and size of embedded JPEG
     my ($jpos, $jlen) = unpack('x84NN', $hdr);
     # check to be sure the JPEG starts in the expected location
-    if ($jpos > 0x94 or $jpos < 0x68) {
+    if ($jpos > 0x94 or $jpos < 0x68 or $jpos & 0x03) {
         $exifTool->Error("Unsupported or corrupted RAF image (version $ver)");
         return 1;
     }
-    # the following RAF version numbers have been tested:
-    # 0100 - E550 V1.00, E900 V1.00, S5600 V1.00, S6000fd V1.00, S6500fd V2.00
-    # 0104 - S5Pro V1.04
-    # 0106 - S5Pro V1.06
-    # 0114 - S9600 V1.00
-    # 0159 - S2Pro V1.00
-    # 0212 - S3Pro V2.12
-    # 0218 - S3Pro V2.18
-    # 0264 - F700  V2.00
-    # 0269 - S9500 V1.02
-    if ($ver !~ /^(0100|0104|0106|0114|0159|0212|0218|0264|0269)$/) {
+    # check to make sure this version of RAF has been tested
+    unless ($testedRAF{$ver}) {
         $exifTool->Error("RAF version $ver not yet tested", 1) and return 1;
     }
     # read the embedded JPEG
@@ -505,7 +551,9 @@ sub WriteRAF($$)
         RAF     => new File::RandomAccess(\$jpeg),
         OutFile => \$outJpeg,
     );
+    $$exifTool{FILE_TYPE} = 'JPEG';
     my $success = $exifTool->WriteJPEG(\%jpegInfo);
+    $$exifTool{FILE_TYPE} = 'RAF';
     unless ($success and $outJpeg) {
         $exifTool->Error("Invalid RAF format");
         return 1;
@@ -562,15 +610,15 @@ sub ProcessRAF($$)
     my ($buff, $jpeg, $warn, $offset);
 
     my $raf = $$dirInfo{RAF};
-    $raf->Read($buff,8) == 8          or return 0;
-    $buff eq 'FUJIFILM'               or return 0;
-    $raf->Seek(0x54, 0)               or return 0;
-    $raf->Read($buff, 8) == 8         or return 0;
-    my ($jpos, $jlen) = unpack('NN', $buff);
+    $raf->Read($buff,0x5c) == 0x5c    or return 0;
+    $buff =~ /^FUJIFILM/              or return 0;
+    my ($jpos, $jlen) = unpack('x84NN', $buff);
     $jpos & 0x8000                   and return 0;
     $raf->Seek($jpos, 0)              or return 0;
     $raf->Read($jpeg, $jlen) == $jlen or return 0;
 
+    $exifTool->FoundTag('RAFVersion', substr($buff, 0x3c, 4));
+
     # extract information from embedded JPEG
     my %dirInfo = (
         Parent => 'RAF',
@@ -628,7 +676,7 @@ FujiFilm maker notes in EXIF information, and to read/write FujiFilm RAW
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
