@@ -20,7 +20,7 @@ use Image::ExifTool::PostScript;
 use Image::ExifTool::XMP qw(EscapeXML UnescapeXML);
 require Exporter;
 
-$VERSION = '1.03';
+$VERSION = '1.09';
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(EscapeHTML UnescapeHTML);
 
@@ -266,12 +266,18 @@ my %entityName; # look up entity names by number (built as necessary)
 #------------------------------------------------------------------------------
 # Convert single UTF-8 character to HTML character reference
 # Inputs: 0) UTF-8 character sequence
-# Returns: HML character reference (ie. "&quot;");
+# Returns: HTML character reference (ie. "&quot;");
 # Note: Must be called via EscapeHTML to load name lookup
 sub EscapeChar($)
 {
     my $ch = shift;
-    my ($val) = ($] >= 5.006001) ? unpack('U0U',$ch) : UnpackUTF8($ch);
+    my $val;
+    if ($] < 5.006001) {
+        ($val) = Image::ExifTool::UnpackUTF8($ch);
+    } else {
+        # the meaning of "U0" is reversed as of Perl 5.10.0!
+        ($val) = unpack($] < 5.010000 ? 'U0U' : 'C0U', $ch);
+    }
     return '?' unless defined $val;
     return "&$entityName{$val};" if $entityName{$val};
     return sprintf('&#x%x;',$val);
@@ -296,7 +302,7 @@ sub EscapeHTML($)
             }
             delete $entityName{39};  # 'apos' is not valid HTML
         }
-        # supress warnings
+        # suppress warnings
         local $SIG{'__WARN__'} = sub { 1 };
         # escape any non-ascii characters for HTML
         $str =~ s/([\xc2-\xf7][\x80-\xbf]+)/EscapeChar($1)/sge;
@@ -326,14 +332,14 @@ sub ProcessHTML($$)
 
     # validate HTML or XHTML file
     $raf->Read($buff, 256) or return 0;
-    $buff =~ /^<(!DOCTYPE HTML|HTML|\?xml)/i or return 0;
-    $buff =~ /<(!DOCTYPE )?HTML/i or return 0 if $1 eq '?xml';
+    $buff =~ /^<(!DOCTYPE\s+HTML|HTML|\?xml)/i or return 0;
+    $buff =~ /<(!DOCTYPE\s+)?HTML/i or return 0 if $1 eq '?xml';
     $exifTool->SetFileType();
 
     $raf->Seek(0,0) or $exifTool->Warn('Seek error'), return 1;
 
-    my $oldsep = Image::ExifTool::PostScript::SetInputRecordSeparator($raf);
-    $oldsep or $exifTool->Warn('Invalid HTML data'), return 1;
+    local $/ = Image::ExifTool::PostScript::GetInputRecordSeparator($raf);
+    $/ or $exifTool->Warn('Invalid HTML data'), return 1;
 
     # extract header information
     my $doc;
@@ -347,6 +353,7 @@ sub ProcessHTML($$)
         $doc .= $buff;
         last if $buff =~ m{</head>}i;
     }
+    return 1 unless defined $doc;
 
     # process all elements in header
     my $tagTablePtr = GetTagTable('Image::ExifTool::HTML::Main');
@@ -411,7 +418,6 @@ sub ProcessHTML($$)
         $val = UnescapeHTML($val);  # unescape HTML character references
         $exifTool->HandleTag($table, $tag, $val);
     }
-    $/ = $oldsep;   # restore original separator
     return 1;
 }
 
@@ -434,7 +440,7 @@ meta information from HTML documents.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
