@@ -10,10 +10,12 @@ package Image::ExifTool::HtmlDump;
 
 use strict;
 use vars qw($VERSION);
+use Image::ExifTool;    # only for FinishTiffDump()
+use Image::ExifTool::HTML qw(EscapeHTML);
 
-$VERSION = '1.14';
+$VERSION = '1.25';
 
-sub DumpTable($$$;$$$$);
+sub DumpTable($$$;$$$$$);
 sub Open($$$;@);
 sub Write($@);
 
@@ -42,6 +44,28 @@ my $htmlHeader2 = <<_END_PART_2_;
 .H { color: #0000ff } /* highlighted tag name */
 .F { color: #aa00dd } /* actual offset differs */
 .M { text-decoration: underline } /* maker notes data */
+.tt { /* tooltip text */
+    visibility: hidden;
+    position: absolute;
+    white-space: nowrap;
+    top: 0;
+    left: 0;
+    font-family: Verdana, sans-serif;
+    font-size: .7em;
+    padding: 2px 4px;
+    border: 1px solid gray;
+    z-index: 3;
+}
+.tb { /* tooltip background */
+    visibility: hidden;
+    position: absolute;
+    background: #ffffdd;
+    opacity: 0.8;
+    -moz-opacity: 0.8;
+    filter: alpha(opacity=80);
+    -ms-filter: 'progid:DXImageTransform.Microsoft.Alpha(Opacity=80)';
+    z-index: 2;
+}
 /* table styles */
 table.dump {
   border-top: 1px solid gray;
@@ -59,8 +83,19 @@ body  { color: black; background: white }
 </style>
 <script language="JavaScript" type="text/JavaScript">
 <!-- Begin
-var t = new Array;
+// tooltip positioning constants
+var TMAR = 4;   // top/left margins
+var BMAR = 16;  // bottom/right margins (scrollbars may overhang inner dimensions)
+var XOFF = 10;  // x offset from cursor
+var YOFF = 40;  // y offset
+var YMIN = 10;  // minimum y offset
+var YTOP = 20;  // y offset when above cursor
+// common variables
+var safari1 = navigator.userAgent.indexOf("Safari/312.6") >= 0;
+var ie6 = navigator.userAgent.toLowerCase().indexOf('msie 6') >= 0;
 var mspan = new Array;
+var hlist, tt, tb;
+
 function GetElementsByClass(classname, tagname) {
   var found = new Array();
   var list = document.getElementsByTagName(tagname);
@@ -77,6 +112,53 @@ function GetElementsByClass(classname, tagname) {
   delete list;
   return found;
 }
+
+// move tooltip
+function move(e)
+{
+  if (!tt) return;
+  if (ie6 && (tt.style.top  == '' || tt.style.top  == 0) &&
+             (tt.style.left == '' || tt.style.left == 0))
+  {
+    tt.style.width  = tt.offsetWidth  + 'px';
+    tt.style.height = tt.offsetHeight + 'px';
+  }
+  var w, h;
+  // browser inconsistencies make getting window size more complex than it should be,
+  // and even then we don't know if it is smaller due to scrollbar width
+  if (typeof(window.innerWidth) == 'number') {
+    w = window.innerWidth;
+    h = window.innerHeight;
+  } else if (document.documentElement && document.documentElement.clientWidth) {
+    w = document.documentElement.clientWidth;
+    h = document.documentElement.clientHeight;
+  } else {
+    w = document.body.clientWidth;
+    h = document.body.clientHeight;
+  }
+  var x = e.clientX + XOFF;
+  var y = e.clientY + YOFF;
+  if (safari1) { // patch for people still using OS X 10.3.9
+    x -= document.body.scrollLeft + document.documentElement.scrollLeft;
+    y -= document.body.scrollTop  + document.documentElement.scrollTop;
+  }
+  var mx = w - BMAR - tt.offsetWidth;
+  var my = h - BMAR - tt.offsetHeight;
+  if (y > my + YOFF - YMIN) y = e.clientY - YTOP - tt.offsetHeight;
+  if (x > mx) x = mx;
+  if (y > my) y = my;
+  if (x < TMAR) x = TMAR;
+  if (y < TMAR) y = TMAR;
+  x += document.body.scrollLeft + document.documentElement.scrollLeft;
+  y += document.body.scrollTop  + document.documentElement.scrollTop;
+  tb.style.width  = tt.offsetWidth  + 'px';
+  tb.style.height = tt.offsetHeight + 'px';
+  tt.style.top  = tb.style.top  = y + 'px';
+  tt.style.left = tb.style.left = x + 'px';
+  tt.style.visibility = tb.style.visibility = 'visible';
+}
+
+// highlight/unhighlight text
 function high(e,on) {
   var targ;
   if (e.target) targ = e.target;
@@ -84,32 +166,43 @@ function high(e,on) {
   if (targ.nodeType == 3) targ = targ.parentNode; // defeat Safari bug
   if (!targ.name) targ = targ.parentNode; // go up another level if necessary
   if (targ.name && document.getElementsByName) {
-    var col;
-    var tip;
+    // un-highlight current objects
+    if (hlist) {
+      for (var i=0; i<hlist.length; ++i) {
+        hlist[i].style.background = 'transparent';
+      }
+      hlist = null;
+    }
+    if (tt) {
+      // hide old tooltip
+      tt.style.visibility = tb.style.visibility = 'hidden';
+      tt = null;
+    }
     if (on) {
-      col = "#ffcc99";
       if (targ.name.substring(0,1) == 't') {
-        var index = parseInt(targ.name.substring(1));
-        tip = t[index];
-        if (tip) delete t[index];
+        // show our tooltip (ID is different than name to avoid confusing IE)
+        tt = document.getElementById('p' + targ.name.substring(1));
+        if (tt) {
+          tb = document.getElementById('tb');
+          move(e);
+        }
       }
-    } else {
-      col = "transparent";
-    }
-    // highlight anchor elements with the same name and add tool tip
-    var list = document.getElementsByName(targ.name);
-    for (var i=0; i<list.length; ++i) {
-      list[i].style.background = col;
-      if (tip) list[i].title += tip;
-    }
-    // use class name to highlight span elements if necessary
-    for (var i=0; i<mspan.length; ++i) {
-      if (mspan[i] != targ.name) continue;
-      list = GetElementsByClass(targ.name, 'span');
-      for (var j=0; j<list.length; ++j) {
-        list[j].style.background = col;
+      // highlight anchor elements with the same name
+      hlist = document.getElementsByName(targ.name);
+      // use class name to highlight span elements if necessary
+      for (var i=0; i<mspan.length; ++i) {
+        if (mspan[i] != targ.name) continue;
+        var slist = GetElementsByClass(targ.name, 'span');
+        // add elements from hlist collection to our array
+        for (var j=0; j<hlist.length; ++j) {
+            slist[slist.length] = hlist[j];
+        }
+        hlist = slist;
+        break;
+      }
+      for (var j=0; j<hlist.length; ++j) {
+        hlist[j].style.background = '#ffcc99';
       }
-      break;
     }
   }
 }
@@ -120,10 +213,11 @@ my $htmlHeader3 = q[
 </script></head>
 <body><noscript><b class=V>--&gt;
 Enable JavaScript for active highlighting and information tool tips!
-</b></noscript><table class=dump cellspacing=0 cellpadding=2>
+</b></noscript>
+<table class=dump cellspacing=0 cellpadding=2>
 <tr><td valign='top'><pre>];
 
-my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)">);
+my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)" onmousemove="move(event)">);
 
 #------------------------------------------------------------------------------
 # New - create new HtmlDump object
@@ -140,7 +234,7 @@ sub new
 # Add information to dump
 # Inputs: 0) HTML dump hash ref, 1) absolute offset in file, 2) data size,
 #         3) comment string, 4) tool tip (or SAME to use previous tip),
-#         5) bit flags (see below)
+#         5) bit flags (see below), 6) true to use same tooltip as last call
 # Bits: 0x01 - print at start of line
 #       0x02 - print red address
 #       0x04 - maker notes data ('M'-class span)
@@ -151,18 +245,21 @@ sub Add($$$$;$$)
     my ($self, $start, $size, $msg, $tip, $flag, $sameTip) = @_;
     my $block = $$self{Block};
     $$block{$start} or $$block{$start} = [ ];
+    my $htip;
     if ($tip and $tip eq 'SAME') {
-        $tip = '';
+        $htip = '';
     } else {
-        $tip = defined $tip ? '\n' . $tip : '';
-        my $m = $msg;
-        $m =~ s/<.*?>//g;       # remove html format codes
-        $tip = "$m$tip";        # add msg as first line in tooltip
+        # use message as first line of tip, and make bold unless in brackets
+        $htip = ($msg =~ /^[[(]/) ? $msg : "<b>$msg</b>";
+        if (defined $tip) {
+            ($tip = EscapeHTML($tip)) =~ s/\n/<br>/g;   # HTML-ize tooltip text
+            $htip .= '<br>' . $tip;
+        }
         # add size if not already done
-        $tip .= "\\n($size bytes)" unless $tip =~ /\\nSize:/;
+        $htip .= "<br>($size bytes)" unless $htip =~ /<br>Size:/;
         ++$self->{TipNum};
     }
-    push @{$$block{$start}}, [ $size, $msg, $tip, $flag, $self->{TipNum} ];
+    push @{$$block{$start}}, [ $size, $msg, $htip, $flag, $self->{TipNum} ];
 }
 
 #------------------------------------------------------------------------------
@@ -170,12 +267,13 @@ sub Add($$$$;$$)
 # Inputs: 0) Dump information hash reference, 1) source file RAF reference,
 #         2) data pointer, 3) data position, 4) output file or scalar reference,
 #         5) limit level (1-3), 6) title
-# Returns: non-zero if useful output was generated
+# Returns: non-zero if useful output was generated,
+#          or -1 on error loading data and ERROR is set to offending data name
 sub Print($$;$$$$$)
 {
     local $_;
     my ($self, $raf, $dataPt, $dataPos, $outfile, $level, $title) = @_;
-    my ($i, $buff, $rtnVal);
+    my ($i, $buff, $rtnVal, $limit, $err);
     my $block = $$self{Block};
     $dataPos = 0 unless $dataPos;
     $outfile = \*STDOUT unless ref $outfile;
@@ -183,7 +281,7 @@ sub Print($$;$$$$$)
     $level or $level = 0;
     my $tell = $raf->Tell();
     my @starts = sort { $a <=> $b } keys %$block;
-    my $pos = $starts[0] || 0;
+    my $pos = 0;
     my $dataEnd = $dataPos + ($dataPt ? length($$dataPt) : 0);
     # initialize member variables
     $$self{Open} = [];
@@ -193,12 +291,13 @@ sub Print($$;$$$$$)
     $$self{Cols} = [ '', '', '', '' ];  # text columns
     # set dump size limits (limits are 4x smaller if bit 0x08 set in flags)
     if ($level <= 1) {
-        $$self{Limit} = 1024;
+        $limit = 1024;
     } elsif ($level <= 2) {
-        $$self{Limit} = 16384;
+        $limit = 16384;
     } else {
-        delete $$self{Limit};   # no limit
+        $limit = 256 * 1024 * 1024; # never dump bigger than 256 MB
     }
+    $$self{Limit} = $limit;
     # pre-initialize open/closed hashes for all columns
     for ($i=0; $i<4; ++$i) {
         $self->{Open}->[$i] = { ID => [ ], Element => { } };
@@ -207,55 +306,92 @@ sub Print($$;$$$$$)
     $bkgStart = $bkgEnd = 0;
     $bkgSpan = '';
     my $index = 0;  # initialize tooltip index
-    my @names;
+    my (@names, $wasUnused);
     for ($i=0; $i<@starts; ++$i) {
         my $start = $starts[$i];
-        my $bytes = $start - $pos;
-        if ($bytes > 0) {
-            if ($pos >= $dataPos and $pos + $bytes <= $dataEnd) {
-                $buff = substr($$dataPt, $pos-$dataPos, $bytes);
+        my $parmList = $$block{$start};
+        my $len = $start - $pos;
+        if ($len > 0 and not $wasUnused) {
+            # we have an unused bytes before this data block
+            --$i;           # dump the data block next time around
+            # split unused data into 2 blocks if it spans end of makernotes
+            if (not defined $wasUnused and $bkgEnd and
+                $pos < $bkgEnd + $dataPos and $pos + $len > $bkgEnd + $dataPos)
+            {
+                $start = $pos;
+                $len = $bkgEnd + $dataPos - $pos;
+                $wasUnused = 0;
             } else {
-                $buff = '';
-                $raf->Seek($pos, 0) and $raf->Read($buff, $bytes);
+                $start = $pos;  # dump the unused bytes now
+                $wasUnused = 1; # avoid re-dumping unused bytes if we get a read error
             }
-            if (length $bytes) {
-                my $str = ($bytes > 1) ? "unused $bytes bytes" : 'pad byte';
-                $self->DumpTable($pos-$dataPos, \$buff, "[$str]", "t$index", 0x108);
-                ++$index;
-            }
-            $pos = $start;  # dumped unused data up to the start of this block
+            my $str = ($len > 1) ? "unused $len bytes" : 'pad byte';
+            $parmList = [ [ $len, "[$str]", undef, 0x108 ] ];
+        } else {
+            undef $wasUnused;
         }
         my $parms;
-        my $parmList = $$block{$start};
         foreach $parms (@$parmList) {
             my ($len, $msg, $tip, $flag, $tipNum) = @$parms;
             next unless $len > 0;
             $flag = 0 unless defined $flag;
             # generate same name for all blocks indexed by this tooltip
-            my $name = $names[$tipNum];
+            my $name;
+            $name = $names[$tipNum] if defined $tipNum;
             my $idx = $index;
             if ($name) {
                 # get index from existing ID
                 $idx = substr($name, 1);
             } else {
-                $name = $names[$tipNum] = "t$index";
+                $name = "t$index";
+                $names[$tipNum] = $name if defined $tipNum;
                 ++$index;
             }
             if ($flag == 4) {
                 $bkgStart = $start - $dataPos;
-                $bkgEnd = $bkgStart + $len;
+                # (allow for nested makernotes data)
+                $bkgEnd = $bkgStart + $len unless $bkgEnd and $bkgEnd > $bkgStart + $len;
                 $bkgSpan = "<span class='$name M'>";
                 push @{$self->{MSpanList}}, $name;
                 next;
             }
-            $tip and $self->{TipList}->[$idx] = $tip;
-            my $end = $start + $len;
-            if ($start >= $dataPos and $end <= $dataEnd) {
-                $buff = substr($$dataPt, $start-$dataPos, $len);
-            } else {
-                $buff = '';
-                $raf->Seek($start, 0) and $raf->Read($buff, $len);
+            # loop until we read the value properly
+            my ($end, $try);
+            for ($try=0; $try<2; ++$try) {
+                $end = $start + $len;
+                # only load as much of the block as we are going to dump
+                my $size = ($len > $limit) ? $limit / 2 : $len;
+                if ($start >= $dataPos and $end <= $dataEnd) {
+                    $buff = substr($$dataPt, $start-$dataPos, $size);
+                    if ($len != $size) {
+                        $buff .= substr($$dataPt, $start-$dataPos+$len-$size, $size);
+                    }
+                } else {
+                    $buff = '';
+                    if ($raf->Seek($start, 0) and $raf->Read($buff, $size) == $size) {
+                        # read end of block
+                        if ($len != $size) {
+                            my $buf2 = '';
+                            unless ($raf->Seek($start+$len-$size, 0) and
+                                    $raf->Read($buf2, $size) == $size)
+                            {
+                                $err = $msg;
+                                # reset $len to the actual length of available data
+                                $raf->Seek(0, 2);
+                                $len = $raf->Tell() - $start;
+                                $tip .= "\nError: Only $len bytes available!" if $tip;
+                                next;
+                            }
+                            $buff .= $buf2;
+                            undef $buf2;
+                        }
+                    } else {
+                        $err = $msg;
+                    }
+                }
+                last;
             }
+            $tip and $self->{TipList}->[$idx] = $tip;
             next unless length $buff;
             # set flag to continue this line if next block is contiguous
             if ($i+1 < @starts and $parms eq $$parmList[-1] and
@@ -265,7 +401,8 @@ sub Print($$;$$$$$)
                 $flag |= 0x100 unless $flag & 0x01 or $nextFlag & 0x01;
             }
             $self->DumpTable($start-$dataPos, \$buff, $msg, $name,
-                             $flag, $pos-$dataPos);
+                             $flag, $len, $pos-$dataPos);
+            undef $buff;
             $pos = $end if $pos < $end;
         }
     }
@@ -280,11 +417,6 @@ sub Print($$;$$$$$)
         for ($i=0; $i<@$mspan; ++$i) {
             Write($outfile, qq(mspan[$i] = "$$mspan[$i]";\n));
         }
-        my $tips = \@{$$self{TipList}};
-        for ($i=0; $i<@$tips; ++$i) {
-            Write($outfile, qq(t[$i] = "$$tips[$i]";\n)) if defined $$tips[$i];
-        }
-        delete $$self{TipList};
         Write($outfile, $htmlHeader3, $self->{Cols}->[0]);
         Write($outfile, '</pre></td><td valign="top">',
                         $preMouse, $self->{Cols}->[1]);
@@ -292,16 +424,28 @@ sub Print($$;$$$$$)
                         $preMouse, $self->{Cols}->[2]);
         Write($outfile, '</pre></td><td valign="top">',
                         $preMouse, $self->{Cols}->[3]);
-        Write($outfile, "</pre></td></tr></table>\n");
+        Write($outfile, "</pre></td></tr></table>\n<div id=tb class=tb> </div>\n");
+        my $tips = \@{$$self{TipList}};
+        for ($i=0; $i<@$tips; ++$i) {
+            my $tip = $$tips[$i];
+            Write($outfile, "<div id=p$i class=tt>$tip</div>\n") if defined $tip;
+        }
+        delete $$self{TipList};
         $rtnVal = 1;
     } else {
         Write($outfile, "$title</title></head><body>\n",
                         "No EXIF or TIFF information found in image\n");
+        $rtnVal = 0;
     }
     Write($outfile, "</body></html>\n");
     for ($i=0; $i<4; ++$i) {
         $self->{Cols}->[$i] = '';   # free memory
     }
+    if ($err) {
+        $err =~ tr/()//d;
+        $$self{ERROR} = $err;
+        return -1;
+    }
     return $rtnVal;
 }
 
@@ -386,13 +530,15 @@ sub Open($$$;@)
 #------------------------------------------------------------------------------
 # Dump a block of data in HTML table form
 # Inputs: 0) HtmlDump object ref, 1) data position, 2) block pointer,
-#         3) message, 4) object name, 5) flag, 6) data end position
-sub DumpTable($$$;$$$$)
+#         3) message, 4) object name, 5) flag, 6) full block length (actual
+#         data may be shorter), 7) data end position
+sub DumpTable($$$;$$$$$)
 {
-    my ($self, $pos, $blockPt, $msg, $name, $flag, $endPos) = @_;
-    my $len = length $$blockPt;
+    my ($self, $pos, $blockPt, $msg, $name, $flag, $len, $endPos) = @_;
+    $len = length $$blockPt unless defined $len;
     $endPos = 0 unless $endPos;
     my ($f0, $dblRef, $id);
+    my $skipped = 0;
     if (($endPos and $pos < $endPos) or $flag & 0x02) {
         # display double-reference addresses in red
         $f0 = "<span class=V>";
@@ -415,7 +561,7 @@ sub DumpTable($$$;$$$$)
             }
             ++$id unless $dblRef;
         }
-        $name = qq{<a name=$name class=$id>};
+        $name = "<a name=$name class=$id>";
         $msg and $msg = "$name$msg</a>";
     } else {
         $name = '';
@@ -446,7 +592,7 @@ sub DumpTable($$$;$$$$)
     for (;;) {
         $self->Open('bkg', ($p>=$bkgStart and $p<$bkgEnd) ? $bkgSpan : '', 1, 2);
         $self->Open('a', $name, 1, 2);
-        my $ch = substr($$blockPt,$p-$pos,1);
+        my $ch = substr($$blockPt,$p-$pos-$skipped,1);
         $c[1] .= sprintf("%.2x", ord($ch));
         # make the character HTML-friendly
         $ch =~ tr/\x00-\x1f\x7f-\xff/./;
@@ -476,6 +622,9 @@ sub DumpTable($$$;$$$$)
         if ($cols < 16) {
             $c[1] .= ($cols == 8 ? '  ' : ' ');
             next;
+        } elsif ($flag & 0x01 and $cols < $len) {
+            $c[1] .= ' ';
+            next;   # put it all on one line
         }
         unless ($$self{Msg}) {
             $c[3] .= $msg;
@@ -498,6 +647,7 @@ sub DumpTable($$$;$$$$)
                     $c[2] .= "     [snip]     \n";
                     $c[3] .= "\n";
                     $p += $n;
+                    $skipped += $len - length $$blockPt;
                 }
             }
         }
@@ -524,6 +674,119 @@ sub DumpTable($$$;$$$$)
     delete $$self{TmpCols};
 }
 
+#------------------------------------------------------------------------------
+# Finish dumping of TIFF image data
+# Inputs: 0) HtmlDump object ref, 1) ExifTool object ref, 2) length of file
+# (this really belongs in Image::ExifTool::Exif, but is placed here so it
+#  is only compiled when needed)
+sub FinishTiffDump($$$)
+{
+    my ($self, $exifTool, $size) = @_;
+    my ($tag, $key, $start, $blockInfo, $i);
+
+    # list of all indirectly referenced TIFF data tags
+    my %offsetPair = (
+        StripOffsets      => 'StripByteCounts',
+        TileOffsets       => 'TileByteCounts',
+        FreeOffsets       => 'FreeByteCounts',
+        ThumbnailOffset   => 'ThumbnailLength',
+        PreviewImageStart => 'PreviewImageLength',
+        JpgFromRawStart   => 'JpgFromRawLength',
+        OtherImageStart   => 'OtherImageLength',
+        ImageOffset       => 'ImageByteCount',
+        AlphaOffset       => 'AlphaByteCount',
+    );
+
+    # add TIFF data to html dump
+    foreach $tag (keys %offsetPair) {
+        my $info = $exifTool->GetInfo($tag);
+        next unless %$info;
+        # Panasonic hack: StripOffsets is not valid for Panasonic RW2 files,
+        # and StripRowBytes is not valid for some RAW images
+        if ($tag eq 'StripOffsets' and $exifTool->{TAG_INFO}{$tag}{PanasonicHack}) {
+            # use RawDataOffset instead if available since it is valid in RW2
+            my $info2 = $exifTool->GetInfo('RawDataOffset');
+            $info2 = $info unless %$info2;
+            my @keys = keys %$info2;
+            my $offset = $$info2{$keys[0]};
+            my $raf = $$exifTool{RAF};
+            # ignore StripByteCounts and assume raw data runs to the end of file
+            if (@keys == 1 and $offset =~ /^\d+$/ and $raf) {
+                my $pos = $raf->Tell();
+                $raf->Seek(0, 2);   # seek to end
+                my $len = $raf->Tell() - $offset;
+                $raf->Seek($pos, 0);
+                if ($len > 0) {
+                    $self->Add($offset, $len, "(Panasonic raw data)", "Size: $len bytes", 0x08);
+                    next;
+                }
+            }
+        }
+        # loop through all offsets tags
+        foreach $key (keys %$info) {
+            my $name = Image::ExifTool::GetTagName($key);
+            my $grp1 = $exifTool->GetGroup($key, 1);
+            my $info2 = $exifTool->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
+            next unless %$info2;
+            my ($key2) = keys %$info2;
+            my $offsets = $$info{$key};
+            my $byteCounts = $$info2{$key2};
+            # (long lists may be SCALAR references)
+            my @offsets = split ' ', (ref $offsets ? $$offsets : $offsets);
+            my @byteCounts = split ' ', (ref $byteCounts ? $$byteCounts : $byteCounts);
+            my $num = scalar @offsets;
+            my $li = 0;
+            my $padBytes = 0;
+            for ($i=0; @offsets and @byteCounts; ++$i) {
+                my $offset = shift @offsets;
+                my $byteCount = shift @byteCounts;
+                my $end = $offset + $byteCount;
+                if (@offsets and @byteCounts) {
+                    # show data as contiguous if only normal pad bytes between blocks
+                    if ($end & 0x01 and $end + 1 == $offsets[0]) {
+                        $end += 1;
+                        ++$padBytes;    # count them
+                    }
+                    if ($end == $offsets[0]) {
+                        # combine these two blocks
+                        $byteCounts[0] += $offsets[0] - $offset;
+                        $offsets[0] = $offset;
+                        next;
+                    }
+                }
+                my $msg = $exifTool->GetGroup($key, 1) . ':' . $tag;
+                $msg =~ s/(Offsets?|Start)$/ /;
+                if ($num > 1) {
+                    $msg .= "$li-" if $li != $i;
+                    $msg .= "$i ";
+                    $li = $i + 1;
+                }
+                $msg .= "data";
+                my $tip = "Size: $byteCount bytes";
+                $tip .= ", incl. $padBytes pad bytes" if $padBytes;
+                $self->Add($offset, $byteCount, "($msg)", $tip, 0x08);
+            }
+        }
+    }
+    # find offset of last dumped information, and dump any unknown trailer
+    my $last = 0;
+    my $block = $$self{Block};
+    foreach $start (keys %$block) {
+        foreach $blockInfo (@{$$block{$start}}) {
+            my $end = $start + $$blockInfo[0];
+            $last = $end if $last < $end;
+        }
+    }
+    my $diff = $size - $last;
+    if ($diff > 0 and ($last or $exifTool->Options('Unknown'))) {
+        if ($diff > 1 or $size & 0x01) {
+            $self->Add($last, $diff, "[unknown data]", "Size: $diff bytes", 0x08);
+        } else {
+            $self->Add($last, $diff, "[trailing pad byte]", undef, 0x08);
+        }
+    }
+}
+
 #------------------------------------------------------------------------------
 # utility routine to write to file or memory
 # Inputs: 0) file or scalar reference, 1-N) list of stuff to write
@@ -575,7 +838,7 @@ display linefeeds in the tool tips.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
