@@ -22,7 +22,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.14';
+$VERSION = '1.17';
 
 sub ProcessICC($$);
 sub ProcessICC_Profile($$$);
@@ -83,6 +83,7 @@ my %profileClass = (
     chad => 'ChromaticAdaptation',
     chrm => {
         Name => 'Chromaticity',
+        Groups => { 1 => 'ICC_Profile#' }, #(just for the group list)
         SubDirectory => {
             TagTable => 'Image::ExifTool::ICC_Profile::Chromaticity',
             Validate => '$type eq "chrm"',
@@ -96,7 +97,14 @@ my %profileClass = (
             Validate => '$type eq "clrt"',
         },
     },
-    cprt => 'ProfileCopyright',
+    clot => { # new in version 4.2
+        Name => 'ColorantTableOut',
+        Binary => 1,
+    },
+    cprt => {
+        Name => 'ProfileCopyright',
+        ValueConv => '$val=~s/\0.*//; $val', # may be null terminated
+    },
     crdi => 'CRDInfo', #2
     dmnd => {
         Name => 'DeviceMfgDesc',
@@ -195,7 +203,7 @@ my %profileClass = (
     mmod => 'MakeAndModel',
     dscm => 'ProfileDescriptionML',
     ndin => 'NativeDisplayInfo',
-    
+
     # Microsoft custom tags (ref http://msdn2.microsoft.com/en-us/library/ms536870.aspx)
     MS00 => 'WCSProfiles',
 
@@ -596,9 +604,9 @@ sub WriteICC_Profile($$;$)
     my $dirName = $$dirInfo{DirName} || 'ICC_Profile';
     # (don't write AsShotICCProfile or CurrentICCProfile here)
     return undef unless $dirName eq 'ICC_Profile';
-    my $newValueHash = $exifTool->GetNewValueHash($Image::ExifTool::Extra{$dirName});
-    return undef unless Image::ExifTool::IsOverwriting($newValueHash);
-    my $val = Image::ExifTool::GetNewValues($newValueHash);
+    my $nvHash = $exifTool->GetNewValueHash($Image::ExifTool::Extra{$dirName});
+    return undef unless Image::ExifTool::IsOverwriting($nvHash);
+    my $val = Image::ExifTool::GetNewValues($nvHash);
     $val = '' unless defined $val;
     ++$exifTool->{CHANGED};
     return $val;
@@ -817,7 +825,7 @@ data created on one device into another device's native color space.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
