@@ -14,7 +14,7 @@ package Image::ExifTool::IPTC;
 use strict;
 use vars qw($VERSION $AUTOLOAD %iptcCharset);
 
-$VERSION = '1.29';
+$VERSION = '1.32';
 
 %iptcCharset = (
     "\x1b%G"  => 'UTF8',
@@ -120,6 +120,7 @@ my %fileFormat = (
     0 => {
         Name => 'EnvelopeRecordVersion',
         Format => 'int16u',
+        Mandatory => 1,
     },
     5 => {
         Name => 'Destination',
@@ -190,8 +191,13 @@ my %fileFormat = (
             UTF-8 character coding is "ESC % G", but this is displayed as "UTF8" for
             convenience.  Either string may be used when writing.  The value of this tag
             affects the decoding of string values in the Application and NewsPhoto
-            records
+            records.  This tag is marked as "unsafe" to prevent it from being copied by
+            default in a group operation because existing tags in the destination image
+            may use a different encoding. When creating a new IPTC record from scratch,
+            it is suggested that this be set to "UTF8" if special characters are a
+            possibility
         },
+        Protected => 1,
         Format => 'string[0,32]',
         # convert ISO 2022 escape sequences to a more readable format
         PrintConv => \&PrintCodedCharset,
@@ -220,6 +226,7 @@ my %fileFormat = (
     0 => {
         Name => 'ApplicationRecordVersion',
         Format => 'int16u',
+        Mandatory => 1,
     },
     3 => {
         Name => 'ObjectTypeReference',
@@ -644,6 +651,7 @@ my %fileFormat = (
     0 => {
         Name => 'NewsPhotoVersion',
         Format => 'int16u',
+        Mandatory => 1,
     },
     10 => {
         Name => 'IPTCPictureNumber',
