@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.03';
+$VERSION = '1.08';
 
 %Image::ExifTool::MPEG::Audio = (
     GROUPS => { 2 => 'Audio' },
@@ -186,6 +186,7 @@ $VERSION = '1.03';
     # Bit 23 - private bit
     'Bit24-25' => {
         Name => 'ChannelMode',
+        RawConv => '$self->{MPEG_Mode} = $val',
         PrintConv => {
             0 => 'Stereo',
             1 => 'Joint Stereo',
@@ -305,6 +306,15 @@ $VERSION = '1.03';
     #'Bit62'    => 'IntraQuantMatrixFlag',
 );
 
+%Image::ExifTool::MPEG::VBR = (
+    GROUPS => { 2 => 'Audio' },
+    VARS => { NO_ID => 1 },
+    NOTES => 'These tags are extracted for variable bitrate audio.',
+    1 => { Name => 'VBRFrames' },
+    2 => { Name => 'VBRBytes' },
+    3 => { Name => 'VBRScale' },
+);
+
 # composite tags
 %Image::ExifTool::MPEG::Composite = (
     Duration => {
@@ -313,23 +323,28 @@ $VERSION = '1.03';
             0 => 'FileSize',
         },
         Desire => {
-            1 => 'AudioBitrate',
-            2 => 'VideoBitrate',
+            1 => 'ID3Size',
+            2 => 'MPEG:AudioBitrate',
+            3 => 'MPEG:VideoBitrate',
+            4 => 'MPEG:VBRFrames',
+            5 => 'MPEG:SampleRate',
+            6 => 'MPEG:MPEGAudioVersion',
         },
-        # calculate duration as file size divided by total bitrate
-        # (note: this is only approximate!)
         ValueConv => q{
-            return undef unless $val[1] or $val[2];
-            return undef if $prt[1] and not $prt[1] =~ /^\d+$/;
-            return undef if $val[2] and not $val[2] =~ /^\d+$/;
-            return (8 * $val[0]) / (($prt[1]||0) + ($val[2]||0));
-        },
-        PrintConv => q{
-            my $h = int($val / 3600);
-            my $m = int($val / 60 - $h * 60);
-            my $s = $val - $h * 3600 - $m * 60;
-            return sprintf("%d:%.2d:%05.2f (approx)",$h,$m,$s);
+            if ($val[4] and defined $val[5] and defined $val[6]) {
+                # calculate from number of VBR audio frames
+                my $mfs = $prt[5] / ($val[6] == 3 ? 144 : 72);
+                # calculate using VBR length
+                return 8 * $val[4] / $mfs;
+            }
+            # calculate duration as file size divided by total bitrate
+            # (note: this is only approximate!)
+            return undef unless $val[2] or $val[3];
+            return undef if $prt[2] and not $prt[2] =~ /^\d+$/;
+            return undef if $val[3] and not $val[3] =~ /^\d+$/;
+            return (8 * ($val[0] - ($val[1]||0))) / (($prt[2]||0) + ($val[3]||0));
         },
+        PrintConv => 'ConvertDuration($val) . " (approx)"',
     },
 );
 
@@ -362,10 +377,11 @@ sub ProcessFrameHeader($$@)
 # Read MPEG audio frame header
 # Inputs: 0) ExifTool object reference, 1) Reference to audio data
 # Returns: 1 on success, 0 if no audio header was found
-sub ProcessMPEGAudio($$)
+sub ParseMPEGAudio($$)
 {
     my ($exifTool, $buffPt) = @_;
-    my $word;
+    my ($word, $pos);
+    my $ext = $$exifTool{FILE_EXT} || '';
 
     for (;;) {
         # find frame sync
@@ -378,12 +394,17 @@ sub ProcessMPEGAudio($$)
         # validate header as much as possible
         if (($word & 0x180000) == 0x080000 or   # 01 is a reserved version ID
             ($word & 0x060000) == 0x000000 or   # 00 is a reserved layer description
+            ($word & 0x00f000) == 0x000000 or   # 0000 is the "free" bitrate index
             ($word & 0x00f000) == 0x00f000 or   # 1111 is a bad bitrate index
-            ($word & 0x000600) == 0x000600 or   # 11 is a reserved sampling frequency
+            ($word & 0x000c00) == 0x000c00 or   # 11 is a reserved sampling frequency
             ($word & 0x000003) == 0x000002)     # 10 is a reserved emphasis
         {
-            return 0;   # invalid frame header
+            # give up easily unless this really should be an MP3 file
+            return 0 unless $ext eq 'MP3';
+            pos($$buffPt) = pos($$buffPt) - 1;
+            next;
         }
+        $pos = pos($$buffPt);
         last;
     }
     # set file type if not done already
@@ -391,6 +412,41 @@ sub ProcessMPEGAudio($$)
 
     my $tagTablePtr = GetTagTable('Image::ExifTool::MPEG::Audio');
     ProcessFrameHeader($exifTool, $tagTablePtr, $word);
+
+    # extract the VBR information (ref MP3::Info)
+    my ($v, $m) = ($$exifTool{MPEG_Vers}, $$exifTool{MPEG_Mode});
+    while (defined $v and defined $m) {
+        my $len = length $$buffPt;
+        $pos += $v == 3 ? ($m == 3 ? 17 : 32) : ($m == 3 ?  9 : 17);
+        last if $pos + 8 > $len;
+        my $buff = substr($$buffPt, $pos, 8);
+        last unless $buff =~ /^(Xing|Info)/;
+        my $vbrTable = GetTagTable('Image::ExifTool::MPEG::VBR');
+        my $flags = unpack('x4N', $buff);
+        $pos += 8;
+        if ($flags & 0x01) {    # VBRFrames
+            last if $pos + 4 > $len;
+            $exifTool->HandleTag($vbrTable, 1, unpack("x${pos}N", $$buffPt));
+            $pos += 4;
+        }
+        if ($flags & 0x02) {    # VBRBytes
+            last if $pos + 4 > $len;
+            $exifTool->HandleTag($vbrTable, 2, unpack("x${pos}N", $$buffPt));
+            $pos += 4;
+        }
+        if ($flags & 0x04) {    # VBR_TOC
+            last if $pos + 100 > $len;
+            # (ignore toc for now)
+            $pos += 100;
+        }
+        if ($flags & 0x08) {    # VBRScale
+            last if $pos + 4 > $len;
+            $exifTool->HandleTag($vbrTable, 3, unpack("x${pos}N", $$buffPt));
+            $pos += 4;
+        }
+        last;
+	}
+
     return 1;
 }
 
@@ -424,18 +480,26 @@ sub ProcessMPEGVideo($$)
 # Read MPEG audio and video frame headers
 # Inputs: 0) ExifTool object reference, 1) Reference to audio/video data
 # Returns: 1 on success, 0 if no video header was found
-sub ProcessMPEGAudioVideo($$)
+sub ParseMPEGAudioVideo($$)
 {
     my ($exifTool, $buffPt) = @_;
-    my %found;
+    my (%found, $didHdr);
     my $rtnVal = 0;
-    my %proc = ( audio => \&ProcessMPEGAudio, video => \&ProcessMPEGVideo );
+    my %proc = ( audio => \&ParseMPEGAudio, video => \&ProcessMPEGVideo );
 
     delete $exifTool->{AudioBitrate};
     delete $exifTool->{VideoBitrate};
 
     while ($$buffPt =~ /\0\0\x01(\xb3|\xc0)/g) {
         my $type = $1 eq "\xb3" ? 'video' : 'audio';
+        unless ($didHdr) {
+            # make sure we didn't miss an audio frame sync before this (ie. MP3 file)
+            # (the last byte of the 4-byte MP3 audio frame header word may be zero,
+            # but the 2nd last must be non-zero, so we need only check to pos-3)
+            my $buff = substr($$buffPt, 0, pos($$buffPt) - 3);
+            $found{audio} = 1 if ParseMPEGAudio($exifTool, \$buff);
+            $didHdr = 1;
+        }
         next if $found{$type};
         my $len = length($$buffPt) - pos($$buffPt);
         last if $len < 4;
@@ -453,7 +517,7 @@ sub ProcessMPEGAudioVideo($$)
 }
 
 #------------------------------------------------------------------------------
-# Read information frame an MPEG file
+# Read information from an MPEG file
 # Inputs: 0) ExifTool object reference, 1) Directory information reference
 # Returns: 1 on success, 0 if this wasn't a valid MPEG file
 sub ProcessMPEG($$)
@@ -469,7 +533,7 @@ sub ProcessMPEG($$)
     $raf->Seek(0,0);
     $raf->Read($buff, 65536*4) or return 0;
 
-    return ProcessMPEGAudioVideo($exifTool, \$buff);
+    return ParseMPEGAudioVideo($exifTool, \$buff);
 }
 
 1;  # end
@@ -496,7 +560,7 @@ based on unofficial sources which may be incomplete, inaccurate or outdated.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -514,6 +578,7 @@ under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<Image::ExifTool::TagNames/MPEG Tags>,
+L<MP3::Info(3pm)|MP3::Info>,
 L<Image::ExifTool(3pm)|Image::ExifTool>
 
 =cut
