@@ -14,8 +14,9 @@ use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Exif;
 
 sub ProcessUnknown($$$);
+sub FixLeicaBase($$;$);
 
-$VERSION = '1.31';
+$VERSION = '1.51';
 
 my $debug;          # set to 1 to enabled debugging code
 
@@ -30,7 +31,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteCanon',
         # (starts with an IFD)
-        Condition => '$self->{CameraMake} =~ /^Canon/',
+        Condition => '$$self{Make} =~ /^Canon/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Canon::Main',
             ByteOrder => 'Unknown',
@@ -41,7 +42,7 @@ my $debug;          # set to 1 to enabled debugging code
         # do negative lookahead assertion just to get tags
         # in a nice order for documentation
         # (starts with an IFD)
-        Condition => '$self->{CameraMake}=~/^CASIO(?! COMPUTER CO.,LTD)/',
+        Condition => '$$self{Make}=~/^CASIO(?! COMPUTER CO.,LTD)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Main',
             ByteOrder => 'Unknown',
@@ -50,16 +51,18 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteCasio2',
         # (starts with "QVC\0")
-        Condition => '$self->{CameraMake}=~/^CASIO COMPUTER CO.,LTD/',
+        # (also found in AVI and MOV videos)
+        Condition => '$$self{Make}=~/^CASIO COMPUTER CO.,LTD/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Type2',
             Start => '$valuePtr + 6',
             ByteOrder => 'Unknown',
+            FixBase => 1, # necessary for AVI and MOV videos
         },
     },
     {
-        # The Fuji programmers really botched this one up,
-        # but with a bit of work we can still read this directory
+        # The Fuji maker notes use a structure similar to a self-contained
+        # TIFF file, but with "FUJIFILM" instead of the standard TIFF header
         Name => 'MakerNoteFujiFilm',
         # (starts with "FUJIFILM" -- also used by some Leica, Minolta and Sharp models)
         Condition => '$$valPt =~ /^FUJIFILM/',
@@ -84,11 +87,9 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteHP2', # PhotoSmart E427
-        Condition => q{
-            $self->{CameraMake} =~ /^Hewlett-Packard/ and
-            $$valPt !~ /^.{8}Hewlett-Packard/s and
-            $$valPt !~ /^IIII/ and $$valPt =~ /^\d{3}.\0/s
-        },
+        # (this type of maker note also used by BenQ, Mustek, Sanyo, Traveler and Vivitar)
+        Condition => '$$valPt =~ /^610[\0-\4]/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::HP::Type2',
             Start => '$valuePtr',
@@ -98,6 +99,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteHP4', # PhotoSmart M627
         Condition => '$$valPt =~ /^IIII\x04\0/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::HP::Type4',
             Start => '$valuePtr',
@@ -107,12 +109,24 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteHP6', # PhotoSmart M425, M525 and M527
         Condition => '$$valPt =~ /^IIII\x06\0/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::HP::Type6',
             Start => '$valuePtr',
             ByteOrder => 'LittleEndian',
         },
     },
+    {
+        Name => 'MakerNoteISL', # (used in Samsung GX20 samples)
+        Condition => '$$valPt =~ /^ISLMAKERNOTE000\0/',
+        # this maker notes starts with a TIFF-like header at offset 0x10
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Unknown::Main',
+            Start => '$valuePtr + 24',
+            Base => '$start - 8',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteJVC',
         Condition => '$$valPt=~/^JVC /',
@@ -124,14 +138,16 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteJVCText',
-        Condition => '$self->{CameraMake}=~/^(JVC|Victor)/ and $$valPt=~/^VER:/',
+        Condition => '$$self{Make}=~/^(JVC|Victor)/ and $$valPt=~/^VER:/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::JVC::Text',
         },
     },
     {
         Name => 'MakerNoteKodak1a',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK INFO/',
+        Condition => '$$self{Make}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK INFO/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Main',
             Start => '$valuePtr + 8',
@@ -140,7 +156,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteKodak1b',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK/',
+        Condition => '$$self{Make}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Main',
             Start => '$valuePtr + 8',
@@ -154,6 +171,7 @@ my $debug;          # set to 1 to enabled debugging code
             $$valPt =~ /^.{8}Eastman Kodak/s or
             $$valPt =~ /^\x01\0[\0\x01]\0\0\0\x04\0[a-zA-Z]{4}/
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type2',
             ByteOrder => 'BigEndian',
@@ -163,7 +181,8 @@ my $debug;          # set to 1 to enabled debugging code
         # not much to key on here, but we know the
         # upper byte of the year should be 0x07:
         Name => 'MakerNoteKodak3',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^.{12}\x07/s',
+        Condition => '$$self{Make}=~/^EASTMAN KODAK/ and $$valPt=~/^.{12}\x07/s',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type3',
             ByteOrder => 'BigEndian',
@@ -171,7 +190,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteKodak4',
-        Condition => '$self->{CameraMake}=~/^Eastman Kodak/ and $$valPt=~/^.{41}JPG/s',
+        Condition => '$$self{Make}=~/^Eastman Kodak/ and $$valPt=~/^.{41}JPG/s',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type4',
             ByteOrder => 'BigEndian',
@@ -180,11 +200,12 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak5',
         Condition => q{
-            $self->{CameraMake}=~/^EASTMAN KODAK/ and
-            ($self->{CameraModel}=~/CX(4200|4230|4300|4310|6200|6230)/ or
+            $$self{Make}=~/^EASTMAN KODAK/ and
+            ($$self{Model}=~/CX(4200|4230|4300|4310|6200|6230)/ or
             # try to pick up similar models we haven't tested yet
             $$valPt=~/^\0(\x1a\x18|\x3a\x08|\x59\xf8|\x14\x80)\0/)
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type5',
             ByteOrder => 'BigEndian',
@@ -193,9 +214,10 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak6a',
         Condition => q{
-            $self->{CameraMake}=~/^EASTMAN KODAK/ and
-            $self->{CameraModel}=~/DX3215/
+            $$self{Make}=~/^EASTMAN KODAK/ and
+            $$self{Model}=~/DX3215/
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type6',
             ByteOrder => 'BigEndian',
@@ -204,17 +226,93 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak6b',
         Condition => q{
-            $self->{CameraMake}=~/^EASTMAN KODAK/ and
-            $self->{CameraModel}=~/DX3700/
+            $$self{Make}=~/^EASTMAN KODAK/ and
+            $$self{Model}=~/DX3700/
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type6',
             ByteOrder => 'LittleEndian',
         },
     },
+    {
+        Name => 'MakerNoteKodak7',
+        # look for something that looks like a serial number
+        # (confirmed serial numbers have the format KXXXX########, but we also
+        #  accept other strings from sample images that may be serial numbers)
+        Condition => q{
+            $$self{Make}=~/Kodak/i and
+            $$valPt =~ /^[CK][A-Z\d]{3} ?[A-Z\d]{1,2}\d{2}[A-Z\d]\d{4}[ \0]/
+        },
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type7',
+            ByteOrder => 'LittleEndian',
+        },
+    },
+    {
+        Name => 'MakerNoteKodak8a',
+        # IFD-format maker notes: look for reasonable number of
+        # entries and check format and count of first IFD entry
+        Condition => q{
+            $$self{Make}=~/Kodak/i and
+            ($$valPt =~ /^\0[\x02-\x7f]..\0[\x01-\x0c]\0\0/s or
+             $$valPt =~ /^[\x02-\x7f]\0..[\x01-\x0c]\0..\0\0/s)
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type8',
+            ProcessProc => \&ProcessUnknown,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteKodak8b',
+        # TIFF-format maker notes
+        Condition => q{
+            $$self{Make}=~/Kodak/i and
+            $$valPt =~ /^(MM\0\x2a\0\0\0\x08|II\x2a\0\x08\0\0\0)/
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type8',
+            ProcessProc => \&ProcessUnknown,
+            ByteOrder => 'Unknown',
+            Start => '$valuePtr + 8',
+            Base => '$start - 8',
+        },
+    },
+    {
+        Name => 'MakerNoteKodak9',
+        # test header and Kodak:DateTimeOriginal
+        Condition => q{
+            $$self{Make}=~/Kodak/i and
+            $$valPt =~ /^IIII.{16}\d{4}\/\d{2}\/\d{2} /s
+        },
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type9',
+            ByteOrder => 'LittleEndian',
+        },
+    },
+    {
+        Name => 'MakerNoteKodak10',
+        # yet another type of Kodak IFD-format maker notes:
+        # this type begins with a byte order indicator,
+        # followed immediately by the IFD
+        Condition => q{
+            $$self{Make}=~/Kodak/i and
+            $$valPt =~ /^(MM\0[\x02-\x7f]|II[\x02-\x7f]\0)/
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type10',
+            ProcessProc => \&ProcessUnknown,
+            ByteOrder => 'Unknown',
+            Start => '$valuePtr + 2',
+        },
+    },
     {
         Name => 'MakerNoteKodakUnknown',
-        Condition => '$self->{CameraMake}=~/Kodak/i',
+        Condition => '$$self{Make}=~/Kodak/i and $$valPt!~/^AOC\0/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Unknown',
             ByteOrder => 'BigEndian',
@@ -223,7 +321,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKyocera',
         # (starts with "KYOCERA")
-        Condition => '$self->{CameraMake}=~/^KYOCERA/',
+        Condition => '$$self{Make}=~/^KYOCERA/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Unknown::Main',
             Start => '$valuePtr + 22',
@@ -235,7 +333,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteMinolta',
         Condition => q{
-            $self->{CameraMake}=~/^(Konica Minolta|Minolta)/i and
+            $$self{Make}=~/^(Konica Minolta|Minolta)/i and
             $$valPt !~ /^(MINOL|CAMER|MLY0|KC|\+M\+M|\xd7)/
         },
         SubDirectory => {
@@ -261,14 +359,15 @@ my $debug;          # set to 1 to enabled debugging code
         # /^+M+M/ - DiMAGE E201
         # /^\xd7/ - DiMAGE RD3000
         Name => 'MakerNoteMinolta3',
-        Condition => '$self->{CameraMake} =~ /^(Konica Minolta|Minolta)/i',
+        Condition => '$$self{Make} =~ /^(Konica Minolta|Minolta)/i',
         Notes => 'not EXIF-based',
         Binary => 1,
+        NotIFD => 1,
     },
     {
         # this maker notes starts with a standard TIFF header at offset 0x0a
         Name => 'MakerNoteNikon',
-        Condition => '$self->{CameraMake}=~/^NIKON/i and $$valPt=~/^Nikon\x00\x02/',
+        Condition => '$$self{Make}=~/^NIKON/i and $$valPt=~/^Nikon\x00\x02/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Main',
             Start => '$valuePtr + 18',
@@ -279,7 +378,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         # older Nikon maker notes
         Name => 'MakerNoteNikon2',
-        Condition => '$self->{CameraMake}=~/^NIKON/ and $$valPt=~/^Nikon\x00\x01/',
+        Condition => '$$self{Make}=~/^NIKON/ and $$valPt=~/^Nikon\x00\x01/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Type2',
             Start => '$valuePtr + 8',
@@ -289,7 +388,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         # headerless Nikon maker notes
         Name => 'MakerNoteNikon3',
-        Condition => '$self->{CameraMake}=~/^NIKON/i',
+        Condition => '$$self{Make}=~/^NIKON/i',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Main',
             ByteOrder => 'Unknown', # most are little-endian, but D1 is big
@@ -310,7 +409,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteOlympus2',
         # new Olympus maker notes start with "OLYMPUS\0"
-        Condition => '$$valPt =~ /^OLYMPUS\0/ and $$self{OlympusType2} = 1',
+        Condition => '$$valPt =~ /^OLYMPUS\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Olympus::Main',
             Start => '$valuePtr + 12',
@@ -321,18 +420,43 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteLeica',
         # (starts with "LEICA\0")
-        Condition => '$self->{CameraMake} =~ /^LEICA/',
+        Condition => '$$self{Make} =~ /^LEICA/',
         SubDirectory => {
-            # Leica uses the same format as Panasonic
+            # many Leica models use the same format as Panasonic
             TagTable => 'Image::ExifTool::Panasonic::Main',
             Start => '$valuePtr + 8',
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteLeica2', # used by the M8
+        # (starts with "LEICA\0")
+        Condition => '$$self{Make} =~ /^Leica Camera AG/ and $$valPt =~ /^LEICA\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Panasonic::Leica2',
+            # (the offset base is different in JPEG and DNG images, but we
+            # can copy makernotes from one to the other, so we need special
+            # logic to decide which base to apply)
+            ProcessProc => \&FixLeicaBase,
+            Start => '$valuePtr + 8',
+            Base => '$start', # (- 8 for DNG images!)
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteLeica3', # used by the R8 and R9
+        # (starts with "LEICA\0")
+        Condition => '$$self{Make} =~ /^Leica Camera AG/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Panasonic::Leica3',
+            Start => '$valuePtr',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNotePanasonic',
         # (starts with "Panasonic\0")
-        Condition => '$self->{CameraMake} =~ /^Panasonic/ and $$valPt!~/^MKE/',
+        Condition => '$$self{Make} =~ /^Panasonic/ and $$valPt!~/^MKE/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Main',
             Start => '$valuePtr + 12',
@@ -342,7 +466,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNotePanasonic2',
         # (starts with "Panasonic\0")
-        Condition => '$self->{CameraMake} =~ /^Panasonic/',
+        Condition => '$$self{Make} =~ /^Panasonic/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Type2',
             ByteOrder => 'LittleEndian',
@@ -350,9 +474,12 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNotePentax',
-        # (if Make is 'PENTAX Corporation', starts with "AOC\0" (also Samsung DX-1S))
-        # (if Make is 'Asahi Optical Co.,Ltd', starts with an IFD)
-        Condition => '$self->{CameraMake}=~/^(PENTAX|AOC|Asahi)/ or $$valPt=~/^AOC\0/',
+        # (starts with "AOC\0", but so does MakerNotePentax3)
+        # (also used by some Samsung models)
+        Condition => q{
+            $$valPt=~/^AOC\0/ and
+            $$self{Model} !~ /^PENTAX Optio ?[34]30RS\s*$/
+        },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Pentax::Main',
             # process as Unknown maker notes because the start offset and
@@ -364,9 +491,46 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNotePentax2',
+        # (starts with an IFD)
+        # Casio-like maker notes used only by the Optio 330 and 430
+        Condition => '$$self{Make}=~/^Asahi/ and $$valPt!~/^AOC\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::Type2',
+            ProcessProc => \&ProcessUnknown,
+            FixBase => 1,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePentax3',
+        # (starts with "AOC\0", like the more common Pentax maker notes)
+        # Casio maker notes used only by the Optio 330RS and 430RS
+        Condition => '$$self{Make}=~/^Asahi/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Casio::Type2',
+            ProcessProc => \&ProcessUnknown,
+            FixBase => 1,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePentax4',
+        # (starts with 3 or 4 digits)
+        # HP2-like text-based maker notes used by Optio E20
+        Condition => '$$self{Make}=~/^PENTAX/ and $$valPt=~/^\d{3}/',
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::Type4',
+            Start => '$valuePtr',
+            ByteOrder => 'LittleEndian',
+        },
+    },
     {
         Name => 'MakerNoteRicoh',
-        Condition => '$self->{CameraMake}=~/^RICOH/ and $$valPt=~/^Ricoh/i',
+        # (my test R50 image starts with "      \x02\x01" - PH)
+        Condition => '$$self{Make}=~/^RICOH/ and $$valPt=~/^(Ricoh|      )/i',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Ricoh::Main',
             Start => '$valuePtr + 8',
@@ -375,7 +539,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteRicohText',
-        Condition => '$self->{CameraMake}=~/^RICOH/',
+        Condition => '$$self{Make}=~/^RICOH/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Ricoh::Text',
             ByteOrder => 'Unknown',
@@ -384,13 +549,19 @@ my $debug;          # set to 1 to enabled debugging code
     {
         # PreviewImage (Samsung Digimax plus some other types of cameras)
         %Image::ExifTool::previewImageTagInfo,
-        Condition => '$$valPt =~ /^\xff\xd8\xff\xdb/',
+        Condition => '$$valPt =~ /^\xff\xd8\xff\xdb/ and $$self{Make}!~/(SONY|SANYO|SIGMA)/i',
         Notes => 'Samsung preview image',
     },
+    {
+        Name => 'MakerNoteSamsung',
+        Condition => '$$valPt =~ /^STMN\d{3}/',
+        Binary => 1,
+        Notes => 'Samsung unknown maker notes',
+    },
     {
         Name => 'MakerNoteSanyo',
         # (starts with "SANYO\0")
-        Condition => '$self->{CameraMake}=~/^SANYO/ and $self->{CameraModel} !~ /^(C4|J\d|S\d)\b/',
+        Condition => '$$self{Make}=~/^SANYO/ and $$self{Model}!~/^(C4|J\d|S\d)\b/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sanyo::Main',
             Validate => '$val =~ /^SANYO/',
@@ -401,7 +572,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteSanyoC4',
         # The C4 offsets are wrong by 12, so they must be fixed
-        Condition => '$self->{CameraMake}=~/^SANYO/ and $self->{CameraModel} =~ /^C4\b/',
+        Condition => '$$self{Make}=~/^SANYO/ and $$self{Model}=~/^C4\b/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sanyo::Main',
             Validate => '$val =~ /^SANYO/',
@@ -413,7 +584,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteSanyoPatch',
         # The J1, J2, J4, S1, S3 and S4 offsets are completely screwy
-        Condition => '$self->{CameraMake}=~/^SANYO/',
+        Condition => '$$self{Make}=~/^SANYO/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sanyo::Main',
             Validate => '$val =~ /^SANYO/',
@@ -425,7 +596,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteSigma',
         # (starts with "SIGMA\0")
-        Condition => '$self->{CameraMake}=~/^(SIGMA|FOVEON)/',
+        Condition => '$$self{Make}=~/^(SIGMA|FOVEON)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sigma::Main',
             Validate => '$val =~ /^(SIGMA|FOVEON)/',
@@ -436,29 +607,47 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteSony',
         # (starts with "SONY DSC \0" or "SONY CAM \0")
-        Condition => '$self->{CameraMake}=~/^SONY/ and $self->{TIFF_TYPE}!~/^(SRF|SR2|DNG)$/',
+        Condition => '$$self{Make}=~/^SONY/ and $$valPt=~/^SONY (DSC|CAM)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sony::Main',
-            # validate the maker note because this is sometimes garbage
-            Validate => 'defined($val) and $val =~ /^SONY (DSC|CAM)/',
             Start => '$valuePtr + 12',
             ByteOrder => 'Unknown',
         },
     },
     {
-        Name => 'MakerNoteSonySRF',
-        Condition => '$self->{CameraMake}=~/^SONY/ and $$valPt =~ /^\x01\x00/',
+        Name => 'MakerNoteSony2',
+        # (starts with "SONY PI\0" -- DSC-S650/S700/S750)
+        Condition => '$$self{Make}=~/^SONY/ and $$valPt=~/^SONY PI\0/ and $$self{OlympusCAMER}=1',
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Sony::SRF',
+            TagTable => 'Image::ExifTool::Olympus::Main',
+            Start => '$valuePtr + 12',
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteSony3',
+        # (starts with "PREMI\0" -- DSC-S45/S500)
+        Condition => '$$self{Make}=~/^SONY/ and $$valPt=~/^(PREMI)\0/ and $$self{OlympusCAMER}=1',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Olympus::Main',
+            Start => '$valuePtr + 8',
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteSony4', # used in SR2 and ARW images
+        Condition => '$$self{Make}=~/^SONY/ and $$valPt!~/^\x01\x00/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Sony::Main',
             Start => '$valuePtr',
             ByteOrder => 'Unknown',
         },
     },
     {
-        Name => 'MakerNoteSonySR2',
-        Condition => '$self->{CameraMake}=~/^SONY/',
+        Name => 'MakerNoteSonySRF',
+        Condition => '$$self{Make}=~/^SONY/',
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Sony::Main',
+            TagTable => 'Image::ExifTool::Sony::SRF',
             Start => '$valuePtr',
             ByteOrder => 'Unknown',
         },
@@ -481,8 +670,7 @@ foreach $tagInfo (@Image::ExifTool::MakerNotes::Main) {
     $$tagInfo{Groups} = { 1 => 'MakerNotes' };
     next unless $$tagInfo{SubDirectory};
     # set up this tag so we can write it
-    $$tagInfo{ValueConv} = '\$val';
-    $$tagInfo{ValueConvInv} = '$val';
+    $$tagInfo{Binary} = 1,
     $$tagInfo{MakerNotes} = 1;
 }
 
@@ -497,8 +685,8 @@ sub GetMakerNoteOffset($)
 {
     my $exifTool = shift;
     # figure out where we expect the value data based on camera type
-    my $make = $exifTool->{CameraMake};
-    my $model = $exifTool->{CameraModel};
+    my $make = $exifTool->{Make};
+    my $model = $exifTool->{Model};
     my ($relative, @offsets);
 
     # normally value data starts 4 bytes after end of directory, so this is the default
@@ -507,8 +695,16 @@ sub GetMakerNoteOffset($)
         # some Canon models (FV-M30, Optura50, Optura60) leave 24 unused bytes
         # at the end of the IFD (2 spare IFD entries?)
         push @offsets, 28 if $model =~ /\b(FV\b|OPTURA)/;
+        # some Canon PowerShot models leave 12 unused bytes
+        push @offsets, 16 if $model =~ /(PowerShot|IXUS|IXY)/;
+    } elsif ($make =~ /^CASIO/) {
+        # Casio AVI and MOV images use no padding, and their JPEG's use 4,
+        # except some models like the EX-S770,Z65,Z70,Z75 and Z700 which use 16
+        push @offsets, $$exifTool{FILE_TYPE} =~ /^(RIFF|MOV)$/ ? 0 : (4, 16);
     } elsif ($make =~ /^KYOCERA/) {
         push @offsets, 12;
+    } elsif ($make =~ /^Leica Camera AG/) {
+        push @offsets, 6; # R8, R9 and M8
     } elsif ($make =~ /^OLYMPUS/ and $model =~ /^E-(1|300|330)\b/) {
         push @offsets, 16;
     } elsif ($make =~ /^OLYMPUS/ and
@@ -516,8 +712,15 @@ sub GetMakerNoteOffset($)
         $model =~ /^(C2500L|C-1Z?|C-5000Z|X-2|C720UZ|C725UZ|C150|C2Z|E-10|E-20|FerrariMODEL2003|u20D|u10D)\b/)
     {
         # no expected offset --> determine offset empirically via FixBase()
-    } elsif ($make =~ /^(Panasonic|SONY|JVC)\b/) {
+    } elsif ($make =~ /^(Panasonic|JVC)\b/) {
         push @offsets, 0;
+    } elsif ($make =~ /^SONY/) {
+        # DSLR and "PREMI" models use an offset of 4
+        if ($model =~ /DSLR/ or $$exifTool{OlympusCAMER}) {
+            push @offsets, 4;
+        } else {
+            push @offsets, 0;
+        }
     } elsif ($make eq 'FUJIFILM') {
         # some models have offset of 6, so allow that too (A345,A350,A360,A370)
         push @offsets, 4, 6;
@@ -604,28 +807,50 @@ sub FixBase($$)
     my $fixBase = $exifTool->Options('FixBase');
     my $setBase = (defined $fixBase and $fixBase ne '') ? 1 : 0;
     my ($fix, $fixedBy);
+
+    # get hash of value block positions
+    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart);
+    return 0 unless %$valBlock;
+    # get sorted list of value offsets
+    my @valPtrs = sort { $a <=> $b } keys %$valBlock;
 #
-# handle special case of Canon maker notes with TIFF trailer containing original offset
+# handle special case of Canon maker notes with TIFF footer containing original offset
 #
-    if ($$exifTool{CameraMake} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
+    if ($$exifTool{Make} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
         my $trailerPos = $dirStart + $$dirInfo{DirLen} - 8;
-        my $trailer = substr($$dataPt, $trailerPos, 8);
-        if ($trailer =~ /^(II\x2a\0|MM\0\x2a)/ and  # check for TIFF trailer
-            substr($trailer,0,2) eq GetByteOrder()) # validate byte ordering
+        my $footer = substr($$dataPt, $trailerPos, 8);
+        if ($footer =~ /^(II\x2a\0|MM\0\x2a)/ and  # check for TIFF footer
+            substr($footer,0,2) eq GetByteOrder()) # validate byte ordering
         {
+            my $oldOffset = Get32u(\$footer, 4);
+            my $newOffset = $dirStart + $dataPos;
             if ($$exifTool{HTML_DUMP}) {
                 my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $trailerPos;
-                $exifTool->HtmlDump($filePos, 8, '[Canon MakerNote trailer]')
+                my $str = sprintf('Original maker note offset: 0x%.4x', $oldOffset);
+                if ($oldOffset != $newOffset) {
+                    $str .= sprintf("\nCurrent maker note offset: 0x%.4x", $newOffset);
+                }
+                $exifTool->HtmlDump($filePos, 8, '[Canon MakerNotes footer]', $str);
             }
             if ($setBase) {
                 $fix = $fixBase;
             } else {
-                my $oldOffset = Get32u(\$trailer, 4);
-                my $newOffset = $dirStart + $dataPos;
                 $fix = $newOffset - $oldOffset;
                 return 0 unless $fix;
+                # Picasa and ACDSee have a bug where they update other offsets without
+                # updating the TIFF footer (PH - 2009/02/25), so test for this case:
+                # validate Canon maker note footer fix by checking offset of last value
+                my $maxPt = $valPtrs[-1] + $$valBlock{$valPtrs[-1]};
+                # compare to end of maker notes, taking 8-byte footer into account
+                my $endDiff = $dirStart + $$dirInfo{DirLen} - ($maxPt - $dataPos) - 8;
+                # ignore footer offset only if end difference is exactly correct
+                # (allow for possible padding byte, although I have never seen this)
+                if (not $endDiff or $endDiff == 1) {
+                    $exifTool->Warn('Canon maker note footer may be invalid (ignored)',1);
+                    return 0;
+                }
             }
-            $exifTool->Warn("Adjusted $dirName base by $fix", 1);
+            $exifTool->Warn("Adjusted $dirName base by $fix",1);
             $$dirInfo{FixedBy} = $fix;
             $$dirInfo{Base} += $fix;
             $$dirInfo{DataPos} -= $fix;
@@ -633,14 +858,11 @@ sub FixBase($$)
         }
     }
 #
-# analyze value offsets to see if they need fixing.  The first task is to
-# determine the minimum valid offset used (this is tricky, because we have
-# to weed out bad offsets written by some cameras)
+# analyze value offsets to see if they need fixing.  The first task is to determine
+# the minimum valid offset used (this is tricky, because we have  to weed out bad
+# offsets written by some cameras)
 #
-    # get hash of value block positions
-    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart);
-    return 0 unless %$valBlock;
-
+    my $minPt = $$dirInfo{MinOffset} = $valPtrs[0]; # if life were simple, this would be it
     my $ifdLen = 2 + 12 * Get16u($$dirInfo{DataPt}, $dirStart);
     my $ifdEnd = $dirStart + $ifdLen;
     my ($relative, @offsets) = GetMakerNoteOffset($exifTool);
@@ -652,10 +874,6 @@ sub FixBase($$)
     my $expected = $dataPos + $ifdEnd + (defined $makeDiff ? $makeDiff : 4);
     $debug and print "$expected expected\n";
 
-    # get sorted list of value offsets
-    my @valPtrs = sort { $a <=> $b } keys %$valBlock;
-    my $minPt = $valPtrs[0];    # if life were simple, this would be it
-
     # zero our counters
     my ($countNeg12, $countZero, $countOverlap) = (0, 0, 0);
     my ($valPtr, $last);
@@ -663,16 +881,16 @@ sub FixBase($$)
         printf("%d - %d (%d)\n", $valPtr, $valPtr + $$valBlock{$valPtr},
                $valPtr - ($last || 0)) if $debug;
         if (defined $last) {
-            my $diff = $valPtr - $last;
-            if ($diff == 0 or $diff == 1) {
+            my $gap = $valPtr - $last;
+            if ($gap == 0 or $gap == 1) {
                 ++$countZero;
-            } elsif ($diff == -12 and not $entryBased) {
+            } elsif ($gap == -12 and not $entryBased) {
                 # you get this when value offsets are relative to the IFD entry
                 ++$countNeg12;
-            } elsif ($diff < 0) {
+            } elsif ($gap < 0) {
                 # any other negative difference indicates overlapping values
                 ++$countOverlap if $valPtr; # (but ignore zero value pointers)
-            } elsif ($diff >= $ifdLen) {
+            } elsif ($gap >= $ifdLen) {
                 # ignore previous minimum if we took a jump to near the expected value
                 # (some values can be stored before the IFD)
                 $minPt = $valPtr if abs($valPtr - $expected) <= 4;
@@ -690,17 +908,19 @@ sub FixBase($$)
         # looks like these offsets are entry-based, so use the offsets
         # which have been correcting for individual entry position
         $entryBased = 1;
-        $verbose and $exifTool->Warn("$dirName offsets are entry-based");
+        $verbose and $exifTool->Warn("$dirName offsets are entry-based",1);
     } else {
-        # calculate difference from normal
+        # calculate offset difference from end of IFD to first value
         $diff = ($minPt - $dataPos) - $ifdEnd;
         $shift = 0;
         $countOverlap and $exifTool->Warn("Overlapping $dirName values",1);
         if ($entryBased) {
-            $exifTool->Warn("$dirName offsets do NOT look entry-based");
+            $exifTool->Warn("$dirName offsets do NOT look entry-based",1);
             undef $entryBased;
             undef $relative;
         }
+        # (used for testing to extract differences)
+        # $exifTool->FoundTag('Diff', $diff);
     }
 #
 # handle entry-based offsets
@@ -786,7 +1006,9 @@ sub LocateIFD($$)
     my ($exifTool, $dirInfo) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
-    my $size = $$dirInfo{DirLen} || ($$dirInfo{DataLen} - $dirStart);
+    # (ignore MakerNotes DirLen since sometimes this is incorrect)
+    my $size = $$dirInfo{DataLen} - $dirStart;
+    my $dirLen = $$dirInfo{DirLen} || $size;
     my $tagInfo = $$dirInfo{TagInfo};
     my $ifdOffsetPos;
     # the IFD should be within the first 32 bytes
@@ -820,7 +1042,16 @@ sub LocateIFD($$)
                 $$dirInfo{Base} += $baseShift;
                 $$dirInfo{DataPos} -= $baseShift;
                 # this is a relative directory if Base depends on $start
-                $$dirInfo{Relative} = 1 if $$subdir{Base} =~ /\$start\b/;
+                if ($$subdir{Base} =~ /\$start\b/) {
+                    $$dirInfo{Relative} = 1;
+                    # hack to fix Leica quirk
+                    if ($$subdir{ProcessProc} and $$subdir{ProcessProc} eq \&FixLeicaBase) {
+                        my $oldStart = $$dirInfo{DirStart};
+                        $$dirInfo{DirStart} = $newStart;
+                        FixLeicaBase($exifTool, $dirInfo);
+                        $$dirInfo{DirStart} = $oldStart;
+                    }
+                }
             }
             # add offset to the start of the directory if necessary
             if ($$subdir{OffsetPt}) {
@@ -842,10 +1073,10 @@ sub LocateIFD($$)
 #
 # scan for something that looks like an IFD
 #
-    if ($size >= 14 + $firstTry) {  # minimum size for an IFD
+    if ($dirLen >= 14 + $firstTry) {  # minimum size for an IFD
         my $offset;
 IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
-            last if $offset + 14 > $size;    # 14 bytes is minimum size for an IFD
+            last if $offset + 14 > $dirLen;    # 14 bytes is minimum size for an IFD
             my $pos = $dirStart + $offset;
 #
 # look for a standard TIFF header (Nikon uses it, others may as well),
@@ -858,7 +1089,7 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
             if (defined $ifdOffsetPos) {
                 # get pointer to IFD
                 my $ptr = Get32u($dataPt, $pos + $ifdOffsetPos);
-                if ($ptr >= $ifdOffsetPos + 4 and $ptr + $offset + 14 <= $size) {
+                if ($ptr >= $ifdOffsetPos + 4 and $ptr + $offset + 14 <= $dirLen) {
                     # shift directory start and shorten dirLen accordingly
                     $$dirInfo{DirStart} += $ptr + $offset;
                     $$dirInfo{DirLen} -= $ptr + $offset;
@@ -898,6 +1129,8 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
                 # allow everything to be zero if not first entry
                 # because some manufacturers pad with null entries
                 next unless $format or $count or $index == 0;
+                # patch for Canon EOS 40D firmware 1.0.4 bug: allow zero format for last entry
+                next if $format==0 and $index==$num-1 and $$exifTool{Model}=~/EOS 40D/;
                 # (would like to verify tag ID, but some manufactures don't
                 #  sort entries in order of tag ID so we don't have much of
                 #  a handle to verify this field)
@@ -915,10 +1148,37 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
     return undef;
 }
 
+#------------------------------------------------------------------------------
+# Fix base offset for Leica maker notes
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success, and updates $dirInfo if necessary for new directory
+sub FixLeicaBase($$;$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirStart = $$dirInfo{DirStart} || 0;
+    # get hash of value block positions
+    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart);
+    if (%$valBlock) {
+        # get sorted list of value offsets
+        my @valPtrs = sort { $a <=> $b } keys %$valBlock;
+        my $numEntries = Get16u($dataPt, $dirStart);
+        my $diff = $valPtrs[0] - ($numEntries * 12 + 4);
+        if ($diff > 8) {
+            $$dirInfo{Base} -= 8;
+            $$dirInfo{DataPos} += 8;
+        }
+    }
+    my $success = 1;
+    if ($tagTablePtr) {
+        $success = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+    }
+    return $success;
+}
+
 #------------------------------------------------------------------------------
 # Process unknown maker notes assuming it is in EXIF IFD format
-# Inputs: 0) ExifTool object reference, 1) reference to directory information
-#         2) pointer to tag table
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success, and updates $dirInfo if necessary for new directory
 sub ProcessUnknown($$$)
 {
@@ -938,7 +1198,7 @@ sub ProcessUnknown($$$)
         $success = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
     } else {
         $exifTool->{UnknownByteOrder} = ''; # indicates we tried but didn't set byte order
-        $exifTool->Warn("Unrecognized $$dirInfo{DirName}");
+        $exifTool->Warn("Unrecognized $$dirInfo{DirName}", 1);
     }
     return $success;
 }
@@ -963,7 +1223,7 @@ maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
