@@ -28,7 +28,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.10';
+$VERSION = '1.20';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
@@ -64,7 +64,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     fRAc => {
         Name => 'FractalParameters',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     gAMA => {
         Name => 'Gamma',
@@ -72,19 +72,19 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     gIFg => {
         Name => 'GIFGraphicControlExtension',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     gIFt => {
         Name => 'GIFPlainTextExtension',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     gIFx => {
         Name => 'GIFApplicationExtension',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     hIST => {
         Name => 'PaletteHistogram',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     iCCP => {
         Name => 'ICC_Profile',
@@ -116,7 +116,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     pCAL => {
         Name => 'PixelCalibration',
-        ValueConv => '\$val',
+        Binary => 1,
     },
     pHYs => {
         Name => 'PhysicalPixel',
@@ -132,7 +132,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     sPLT => {
         Name => 'SuggestedPalette',
-        ValueConv => '\$val',
+        Binary => 1,
         PrintConv => 'split("\0",$$val,1)', # extract palette name
     },
     sRGB => {
@@ -152,9 +152,16 @@ $Image::ExifTool::PNG::colorType = -1;
     tIME => {
         Name => 'ModifyDate',
         Groups => { 2 => 'Time' },
+        Writable => 1,
         Shift => 'Time',
         ValueConv => 'sprintf("%.4d:%.2d:%.2d %.2d:%.2d:%.2d", unpack("nC5", $val))',
+        ValueConvInv => q{
+            my @a = ($val=~/^(\d+):(\d+):(\d+)\s+(\d+):(\d+):(\d+)/);
+            @a == 6 or warn('Invalid date'), return undef;
+            return pack('nC5', @a);
+        },
         PrintConv => '$self->ConvertDateTime($val)',
+        PrintConvInv => '$self->InverseDateTime($val)',
     },
     tRNS => {
         Name => 'Transparency',
@@ -165,6 +172,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     tXMP => {
         Name => 'XMP',
+        Notes => 'obsolete location specified by older XMP draft',
         SubDirectory => { TagTable => 'Image::ExifTool::XMP::Main' },
     },
     zTXt => {
@@ -263,10 +271,12 @@ listed below are the only ones that can be written (unless new user-defined
 tags are added via the configuration file), however ExifTool will extract
 any other TextualData tags that are found.
 
-The information for the TextualData tags may be stored as tEXt, zTXt or iTXt
-chunks in the PNG image.  ExifTool will read and edit tags in their original
-form, but only tEXt chunks are written when creating new tags (except for
-the profiles which are written as zTXt if Compress::Zlib is available).
+Data for the TextualData tags may be stored as tEXt, zTXt or iTXt chunks in
+the PNG image.  ExifTool will read and edit tags in their original form, but
+new string tags are created as uncompressed tEXt by default, or as
+compressed zTXt if the -z (Compress) option is used and Compress::Zlib is
+available. Raw profile information is always created as compressed zTXt if
+Compress::Zlib is available.
 
 Some of the tags below are not registered as part of the PNG specification,
 but are included here because they are generated by other software such as
@@ -298,6 +308,16 @@ ImageMagick.
     Model       => { %unreg, Groups => { 2 => 'Camera' } },
     TimeStamp   => { %unreg, Groups => { 2 => 'Time' }, Shift => 'Time' },
     URL         => { %unreg },
+   'XML:com.adobe.xmp' => {
+        Name => 'XMP',
+        Notes => q{
+            location according to the XMP specification -- this is where ExifTool will
+            add a new XMP chunk if the image didn't already contain XMP
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::XMP::Main',
+        },
+    },
    'Raw profile type APP1' => [
         {
             # EXIF table must come first because we key on this in ProcessProfile()
@@ -371,7 +391,7 @@ sub AUTOLOAD
 sub FoundPNG($$$$;$$)
 {
     my ($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff) = @_;
-    my $wasCompressed;
+    my ($wasCompressed, $deflateErr);
     return 0 unless defined $val;
 #
 # First, uncompress data if requested
@@ -381,8 +401,8 @@ sub FoundPNG($$$$;$$)
     my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag) ||
                   # (some software forgets to capitalize first letter)
                   $exifTool->GetTagInfo($tagTablePtr, ucfirst($tag));
+
     if ($compressed and $compressed > 1) {
-        my $warn;
         if ($compressed == 2) { # Inflate/Deflate compression
             if (eval 'require Compress::Zlib') {
                 my ($v2, $stat);
@@ -393,37 +413,38 @@ sub FoundPNG($$$$;$$)
                     $compressed = 0;
                     $wasCompressed = 1;
                 } else {
-                    $warn = "Error inflating $tag";
+                    $deflateErr = "Error inflating $tag";
                 }
             } elsif (not $noCompressLib) {
                 $noCompressLib = 1;
                 my $verb = $outBuff ? 'write' : 'decode';
-                $warn = "Install Compress::Zlib to $verb compressed binary data";
+                $deflateErr = "Install Compress::Zlib to $verb compressed information";
+            } else {
+                $deflateErr = '';   # flag deflate error but no warning
             }
         } else {
             $compressed -= 2;
-            $warn = "Unknown compression method $compressed for $tag";
+            $deflateErr = "Unknown compression method $compressed for $tag";
         }
         if ($compressed and $verbose and $tagInfo and $$tagInfo{SubDirectory}) {
             $exifTool->VerboseDir("Unable to decompress $$tagInfo{Name}", 0, length($val));
         }
-        $warn and $exifTool->Warn($warn);
+        $exifTool->Warn($deflateErr) if $deflateErr and not $outBuff;
     }
 #
 # extract information from subdirectory if available
 #
     if ($tagInfo) {
+        my $tagName = $$tagInfo{Name};
         my $processed;
         if ($$tagInfo{SubDirectory} and not $compressed) {
             my $len = length $val;
             if ($verbose and $exifTool->{INDENT} ne '  ') {
                 if ($wasCompressed and $verbose > 2) {
-                    my $name = $$tagInfo{Name};
+                    my $name = $tagName;
                     $wasCompressed and $name = "Decompressed $name";
                     $exifTool->VerboseDir($name, 0, $len);
-                    my %parms = ( Prefix => $exifTool->{INDENT}, Out => $out );
-                    $parms{MaxLen} = 96 unless $verbose > 3;
-                    Image::ExifTool::HexDump(\$val, undef, %parms);
+                    $exifTool->VerboseDump(\$val);
                 }
                 # don't indent next directory (since it is really the same data)
                 $exifTool->{INDENT} =~ s/..$//;
@@ -438,17 +459,20 @@ sub FoundPNG($$$$;$$)
                 DirStart => 0,
                 DataLen => $len,
                 DirLen => $len,
-                DirName => $$tagInfo{Name},
+                DirName => $tagName,
                 TagInfo => $tagInfo,
+                ReadOnly => 1, # (only used by WriteXMP)
                 OutBuff => $outBuff,
             );
             # no need to re-decompress if already done
             undef $processProc if $wasCompressed and $processProc eq \&ProcessPNG_Compressed;
             # rewrite this directory if necessary (but always process TextualData normally)
             if ($outBuff and not $processProc and $subTable ne \%Image::ExifTool::PNG::TextualData) {
-                return 1 unless $exifTool->{EDIT_DIRS}->{$$tagInfo{Name}};
-                delete $exifTool->{ADD_DIRS}->{$$tagInfo{Name}};
+                return 1 unless $exifTool->{EDIT_DIRS}->{$tagName};
                 $$outBuff = $exifTool->WriteDirectory(\%subdirInfo, $subTable);
+                # if this was an XMP directory, we must make it read-only
+                $tagName eq 'XMP' and Image::ExifTool::XMP::ValidateXMP(\$outBuff,'r');
+                delete $exifTool->{ADD_DIRS}->{$tagName};
             } else {
                 $processed = $exifTool->ProcessDirectory(\%subdirInfo, $subTable, $processProc);
             }
@@ -461,35 +485,57 @@ sub FoundPNG($$$$;$$)
             {
                 # write new value for this tag if necessary
                 my ($isOverwriting, $newVal);
-                if ($exifTool->{DEL_GROUP} and $exifTool->{DEL_GROUP}->{PNG}) {
+                if ($exifTool->{DEL_GROUP}->{PNG}) {
+                    # remove this tag now, but keep in ADD_PNG list to add back later
                     $isOverwriting = 1;
                 } else {
-                    my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-                    $isOverwriting = Image::ExifTool::IsOverwriting($newValueHash, $val);
-                    $newVal = Image::ExifTool::GetNewValues($newValueHash);
+                    # remove this from the list of PNG tags to add
+                    delete $exifTool->{ADD_PNG}->{$tag};
+                    # (also handle case of tEXt tags written with lowercase first letter)
+                    delete $exifTool->{ADD_PNG}->{ucfirst($tag)};
+                    my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+                    $isOverwriting = Image::ExifTool::IsOverwriting($nvHash);
+                    if (defined $deflateErr) {
+                        $newVal = Image::ExifTool::GetNewValues($nvHash);
+                        # can only write tag now if unconditionally deleting it
+                        if ($isOverwriting > 0 and not defined $newVal) {
+                            $val = '<deflate error>';
+                        } else {
+                            $isOverwriting = 0; # can't rewrite this compressed text
+                            $exifTool->Warn($deflateErr) if $deflateErr;
+                        }
+                    } else {
+                        if ($isOverwriting < 0) {
+                            $isOverwriting = Image::ExifTool::IsOverwriting($nvHash, $val);
+                        }
+                        # (must get new value after IsOverwriting() in case it was shifted)
+                        $newVal = Image::ExifTool::GetNewValues($nvHash);
+                    }
                 }
                 if ($isOverwriting) {
                     $$outBuff =  (defined $newVal) ? $newVal : '';
                     ++$exifTool->{CHANGED};
-                    if ($verbose > 1) {
-                        print $out "    - $$tagInfo{Name} = '$val'\n";
-                        print $out "    + $$tagInfo{Name} = '$newVal'\n" if defined $newVal;
-                    }
+                    $exifTool->VerboseValue("- PNG:$tagName", $val);
+                    $exifTool->VerboseValue("+ PNG:$tagName", $newVal) if defined $newVal;
                 }
             }
-            if ($$outBuff and $wasCompressed) {
-                # re-compress the output data
-                my $deflate;
-                if (eval 'require Compress::Zlib') {
-                    my $deflate = Compress::Zlib::deflateInit();
-                    if ($deflate) {
-                        $$outBuff = $deflate->deflate($$outBuff);
-                        $$outBuff .= $deflate->flush() if defined $$outBuff;
-                    } else {
-                        undef $$outBuff;
+            if ($$outBuff) {
+                if ($wasCompressed) {
+                    # re-compress the output data
+                    my $deflate;
+                    if (eval 'require Compress::Zlib') {
+                        my $deflate = Compress::Zlib::deflateInit();
+                        if ($deflate) {
+                            $$outBuff = $deflate->deflate($$outBuff);
+                            $$outBuff .= $deflate->flush() if defined $$outBuff;
+                        } else {
+                            undef $$outBuff;
+                        }
                     }
+                    $$outBuff or $exifTool->Warn("PNG:$tagName not written (compress error)");
+                } elsif ($exifTool->Options('Compress')) {
+                    $exifTool->Warn("PNG:$tagName not compressed (uncompressed tag existed)", 1);
                 }
-                $$outBuff or $exifTool->Warn("Error compressing $$tagInfo{Name} -- unchanged");
             }
             return 1;
         }
@@ -499,7 +545,7 @@ sub FoundPNG($$$$;$$)
         ($name = $tag) =~ s/\s+(.)/\u$1/g;   # remove white space from tag name
         $tagInfo = { Name => $name };
         # make unknown profiles binary data type
-        $$tagInfo{ValueConv} = '\$val' if $tag =~ /^Raw profile type /;
+        $$tagInfo{Binary} = 1 if $tag =~ /^Raw profile type /;
         Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
     }
 #
@@ -537,6 +583,7 @@ sub ProcessProfile($$$)
     my $dataPt = $$dirInfo{DataPt};
     my $tagInfo = $$dirInfo{TagInfo};
     my $outBuff = $$dirInfo{OutBuff};
+    my $tagName = $$tagInfo{Name};
 
     # ImageMagick 5.3.6 writes profiles with the following headers:
     # "\nICC Profile\n", "\nIPTC profile\n", "\n\xaa\x01{generic prof\n"
@@ -547,19 +594,14 @@ sub ProcessProfile($$$)
     my $buff = pack('H*', join('',split(' ',$3)));
     my $actualLen = length $buff;
     if ($len ne $actualLen) {
-        $exifTool->Warn("$$tagInfo{Name} is wrong size (should be $len bytes but is $actualLen)");
+        $exifTool->Warn("$tagName is wrong size (should be $len bytes but is $actualLen)");
         $len = $actualLen;
     }
     my $verbose = $exifTool->Options('Verbose');
     if ($verbose) {
         if ($verbose > 2) {
-            $exifTool->VerboseDir("Decoded $$tagInfo{Name}", 0, $len);
-            my %parms = (
-                Prefix => $exifTool->{INDENT},
-                Out => $exifTool->Options('TextOut'),
-            );
-            $parms{MaxLen} = 96 unless $verbose > 3;
-            Image::ExifTool::HexDump(\$buff, undef, %parms);
+            $exifTool->VerboseDir("Decoded $tagName", 0, $len);
+            $exifTool->VerboseDump(\$buff);
         }
         # don't indent next directory (since it is really the same data)
         $exifTool->{INDENT} =~ s/..$//;
@@ -582,11 +624,11 @@ sub ProcessProfile($$$)
         # process non-EXIF and non-APP1 profile as-is
         if ($outBuff) {
             # no need to rewrite this if not editing tags in this directory
-            my $dir = $$tagInfo{Name};
+            my $dir = $tagName;
             $dir =~ s/_Profile// unless $dir =~ /^ICC/;
             return 1 unless $$editDirs{$dir};
-            delete $$addDirs{$dir};
             $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+            delete $$addDirs{$dir};
         } else {
             $processed = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
         }
@@ -596,14 +638,13 @@ sub ProcessProfile($$$)
         my $hdrLen = length($Image::ExifTool::exifAPP1hdr);
         $dirInfo{DirStart} += $hdrLen;
         $dirInfo{DirLen} -= $hdrLen;
-        $processed = $exifTool->ProcessTIFF(\%dirInfo);
         if ($outBuff) {
-            if ($$outBuff) {
-                $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
-            } else {
-                $$outBuff = '' if $processed;
-            }
+            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr,
+                                                  \&Image::ExifTool::WriteTIFF);
+            $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
             delete $$addDirs{IFD0};
+        } else {
+            $processed = $exifTool->ProcessTIFF(\%dirInfo);
         }
     } elsif ($buff =~ /^$Image::ExifTool::xmpAPP1hdr/) {
         # APP1 XMP information
@@ -613,23 +654,22 @@ sub ProcessProfile($$$)
         $dirInfo{DirLen} -= $hdrLen;
         if ($outBuff) {
             return 1 unless $$editDirs{XMP};
-            delete $$addDirs{XMP};
             $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
             $$outBuff and $$outBuff = $Image::ExifTool::xmpAPP1hdr . $$outBuff;
+            delete $$addDirs{XMP};
         } else {
             $processed = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
         }
     } elsif ($buff =~ /^(MM\0\x2a|II\x2a\0)/) {
         # TIFF information (haven't seen this, but what the heck...)
         return 1 if $outBuff and not $$editDirs{IFD0};
-        $processed = $exifTool->ProcessTIFF(\%dirInfo);
         if ($outBuff) {
-            if ($$outBuff) {
-                $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
-            } else {
-                $$outBuff = '' if $processed;
-            }
+            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr,
+                                                  \&Image::ExifTool::WriteTIFF);
+            $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
             delete $$addDirs{IFD0};
+        } else {
+            $processed = $exifTool->ProcessTIFF(\%dirInfo);
         }
     } else {
         my $profName = $profileType;
@@ -684,10 +724,7 @@ sub ProcessPNG_tEXt($$$)
     my $outBuff = $$dirInfo{OutBuff};
     my $rtnVal = FoundPNG($exifTool, $tagTablePtr, $tag, $val, undef, $outBuff);
     # add header back onto this chunk if we are writing
-    if ($outBuff) {
-        $$outBuff = $tag . "\0" . $$outBuff if $$outBuff;
-        delete $exifTool->{ADD_PNG}->{ucfirst($tag)};
-    }
+    $$outBuff = $tag . "\0" . $$outBuff if $outBuff and $$outBuff;
     return $rtnVal;
 }
 
@@ -707,7 +744,7 @@ sub ProcessPNG_iTXt($$$)
     my $outBuff = $$dirInfo{OutBuff};
     my $rtnVal = FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff);
     if ($outBuff and $$outBuff) {
-        $$outBuff = substr($dat, 0, 2) . "$lang\0$trans\0" . $$outBuff;
+        $$outBuff = $tag . "\0" . substr($dat, 0, 2) . "$lang\0$trans\0" . $$outBuff;
     }
     return $rtnVal;
 }
@@ -730,9 +767,12 @@ sub ProcessPNG($$)
     return 0 unless $raf->Read($sig,8) == 8 and $pngLookup{$sig};
     if ($outfile) {
         Write($outfile, $sig) or $err = 1 if $outfile;
-        # can only add tags in TextualData
-        $exifTool->{ADD_PNG} = $exifTool->GetNewTagInfoHash(\%Image::ExifTool::PNG::TextualData);
-        $exifTool->InitWriteDirs('JPEG');   # use same directories as JPEG
+        # can only add tags in Main and TextualData tables
+        $exifTool->{ADD_PNG} = $exifTool->GetNewTagInfoHash(
+            \%Image::ExifTool::PNG::Main,
+            \%Image::ExifTool::PNG::TextualData);
+        # initialize with same directories as JPEG, but PNG tags take priority
+        $exifTool->InitWriteDirs('JPEG','PNG');
     }
     my ($fileType, $hdrChunk, $endChunk) = @{$pngLookup{$sig}};
     $exifTool->SetFileType($fileType);  # set the FileType tag
@@ -796,6 +836,8 @@ sub ProcessPNG($$)
         unless ($foundHdr) {
             if ($chunk eq $hdrChunk) {
                 $foundHdr = 1;
+            } elsif ($hdrChunk eq 'IHDR' and $chunk eq 'CgBI') {
+                $exifTool->Warn('Non-standard PNG image (Apple iPhone format)');
             } else {
                 $exifTool->Warn("$fileType image did not start with $hdrChunk");
                 last;
@@ -805,19 +847,13 @@ sub ProcessPNG($$)
             # check CRC when in verbose mode (since we don't care about speed)
             my $crc = CalculateCRC(\$hbuf, undef, 4);
             $crc = CalculateCRC(\$dbuf, $crc);
-            unless ($crc == unpack('N',$cbuf)) {
-                $exifTool->Warn("Bad CRC for $chunk chunk");
-            }
+            $crc == unpack('N',$cbuf) or $exifTool->Warn("Bad CRC for $chunk chunk");
             if ($datChunk) {
                 Write($outfile, $hbuf, $dbuf, $cbuf) or $err = 1 if $outfile;
                 next;
             }
             print $out "$fileType $chunk ($len bytes):\n";
-            if ($verbose > 2) {
-                my %dumpParms = ( Out => $out );
-                $dumpParms{MaxLen} = 96 if $verbose <= 4;
-                Image::ExifTool::HexDump(\$dbuf, undef, %dumpParms);
-            }
+            $exifTool->VerboseDump(\$dbuf, Addr => $raf->Tell() - $len - 4) if $verbose > 2;
         }
         # only extract information from chunks in our tables
         my ($theBuff, $outBuff);
@@ -864,7 +900,7 @@ and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
