@@ -273,14 +273,7 @@ sub WritePDF($$)
     if ($buff =~ /$endComment(\d+)\s+(startxref\s+\d+\s+%%EOF\s+)?$/s) {
         $prevUpdate = $1;
         # rewrite the file up to the original EOF
-        my $size = $prevUpdate;
-        for (;;) {
-            last unless $size;
-            my $n = $size > 65536 ? 65536 : $size;
-            $raf->Read($buff, $n) == $n or $rtn = -1, last;
-            Write($outfile, $buff) or $rtn = -1;
-            $size -= $n;
-        }
+        Image::ExifTool::CopyBlock($raf, $outfile, $prevUpdate) or $rtn = -1;
         # verify that we are now at the start of an ExifTool update
         unless ($raf->Read($buff, length $beginComment) and $buff eq $beginComment) {
             $exifTool->Error('Previous ExifTool update is corrupted');
@@ -347,15 +340,14 @@ sub WritePDF($$)
     $keyExt = $$infoRef;
 
     # must set line separator before calling WritePDFValue()
-    my $oldSep = $/;
-    $/ = $capture{newline};
+    local $/ = $capture{newline};
 
     # rewrite PDF Info tags
     my $newTags = $exifTool->GetNewTagInfoHash(\%Image::ExifTool::PDF::Info);
     my $tagID;
     foreach $tagID (sort keys %$newTags) {
         my $tagInfo = $$newTags{$tagID};
-        my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
+        my $nvHash = $exifTool->GetNewValueHash($tagInfo);
         my (@vals, $deleted);
         my $tag = $$tagInfo{Name};
         my $val = $$info{$tag};
@@ -372,9 +364,9 @@ sub WritePDF($$)
                 $val = shift @oldVals;
             }
             for (;;) {
-                if (Image::ExifTool::IsOverwriting($newValueHash, $val) > 0) {
+                if (Image::ExifTool::IsOverwriting($nvHash, $val) > 0) {
                     $deleted = 1;
-                    print $out "    - PDF:$tag = '",$exifTool->Printable($val),"'\n" if $out;
+                    $exifTool->VerboseValue("- PDF:$tag", $val);
                     ++$infoChanged;
                 } else {
                     push @vals, $val;
@@ -390,13 +382,13 @@ sub WritePDF($$)
         next unless $deleted or $$tagInfo{List} or not exists $$infoDict{$tagID};
 
         # add new values to existing ones
-        my @newVals = Image::ExifTool::GetNewValues($newValueHash);
+        my @newVals = Image::ExifTool::GetNewValues($nvHash);
         if (@newVals) {
             push @vals, @newVals;
             ++$infoChanged;
             if ($out) {
                 foreach $val (@newVals) {
-                    print $out "    + PDF:$tag = '",$exifTool->Printable($val),"'\n";
+                    $exifTool->VerboseValue("+ PDF:$tag", $val);
                 }
             }
         }
@@ -453,7 +445,7 @@ sub WritePDF($$)
         DataPt => $$info{XMP},
         Parent => 'PDF',
     );
-    my $xmpTable = GetTagTable('Image::ExifTool::XMP::Main');
+    my $xmpTable = Image::ExifTool::GetTagTable('Image::ExifTool::XMP::Main');
     my $oldChanged = $$exifTool{CHANGED};
     my $newXMP = $exifTool->WriteDirectory(\%xmpInfo, $xmpTable);
     $newXMP = $$info{XMP} ? ${$$info{XMP}} : '' unless defined $newXMP;
@@ -502,7 +494,6 @@ sub WritePDF($$)
     my $rootRef = $$mainDict{Root};
     unless ($rootRef) {
         $exifTool->Error("Can't find Root dictionary");
-        $/ = $oldSep;
         return $rtn;
     }
     if (not $rootChanged and $prevUpdate) {
@@ -644,8 +635,8 @@ sub WritePDF($$)
                 # write this (contiguous-numbered object) section of the xref table
                 Write($outfile, $startID, ' ', $id - $startID + 1, $/, $buff) or $rtn = -1;
             }
-     
-            # write main (trailer) dictionary        
+
+            # write main (trailer) dictionary
             Write($outfile, 'trailer') or $rtn = -1;
             WriteObject($outfile, $mainDict) or $rtn = -1;
         }
@@ -665,7 +656,6 @@ sub WritePDF($$)
     }
     undef $newTool;
     undef %capture;
-    $/ = $oldSep;   # restore original separator
     return $rtn;
 }
 
@@ -695,7 +685,7 @@ C<PDF-update> pseudo group).
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
